<?php
/**
 * Plugin Name: AutomateWoo
 * Plugin URI: http://automatewoo.com
 * Description: Powerful marketing automation for your WooCommerce store.
 * Version: 2.3.3
 * Author: Daniel Bitzer
 * Author URI: http://danielbitzer.com
 * License: GPLv3
 * License URI: http://www.gnu.org/licenses/gpl-3.0
 * Text Domain: automatewoo
 * Domain Path: /languages
 */

// Copyright (c) 2015 Daniel Bitzer. All rights reserved.
//
// Released under the GPLv3 license
// http://www.gnu.org/licenses/gpl-3.0
//
// **********************************************************************
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// **********************************************************************

if( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'AutomateWoo' ) ) :

class AutomateWoo
{
	public $version = '2.3.3';

	public $plugin_prefix = 'aw_';
	public $plugin_url;
	public $plugin_path;
	public $lib_path;
	public $plugin_slug;
	public $plugin_main_file;
	public $plugin_basename;
	public $assets_url;

	public $website_url = 'http://automatewoo.com/';
	public $url_license_docs = 'http://automatewoo.com/docs/licenses/';

	public $post_type = 'aw_workflow';

	/**
	 * Database table names
	 */
	public $table_name_abandoned_cart = 'automatewoo_abandoned_carts';
	public $table_name_guests = 'automatewoo_guests';
	public $table_name_queue = 'automatewoo_queue';
	public $table_name_logs = 'automatewoo_logs';
	public $table_name_log_meta = 'automatewoo_log_meta';
	public $table_name_unsubscribes = 'automatewoo_unsubscribes';


	/**
	 * @var array
	 */
	public $registered_triggers = array();

	/**
	 * @var array
	 */
	public $registered_actions = array();

	/**
	 * @var AW_Admin
	 */
	public $admin;

	/**
	 * @var AW_Admin_Helper
	 */
	public $admin_helper;

	/**
	 * @var AW_Email
	 */
	public $email;

	/**
	 * @var AW_Licence
	 */
	public $licence;

	/**
	 * @var AW_Wishlist
	 */
	public $wishlist;

	/**
	 * @var AW_Abandoned_Cart_Manager
	 */
	public $abandoned_cart;

	/**
	 * @var AW_Session_Tracker
	 */
	public $session_tracker;

	/**
	 * @var AW_Integrations
	 */
	public $integrations;

	/**
	 * @var AW_Options
	 */
	public $options;

	/**
	 * @var AW_Queue_Manager
	 */
	public $queue_manager;

	/**
	 * @var AW_Conversion_Manager
	 */
	public $conversion_manager;

	/**
	 * @var AW_Language_Helper
	 */
	public $language_helper;

	/**
	 * @var AW_Cache_Helper
	 */
	public $cache;

	/**
	 * @var AW_System_Checker
	 */
	public $system_checker;

	/**
	 * @var bool
	 */
	public $debug = false;

	/**
	 * @var AutomateWoo
	 */
	protected static $_instance = null;



	/**
	 * Constructor
	 */
	public function __construct()
	{
		$this->plugin_basename = plugin_basename( __FILE__ );
		list ( $this->plugin_slug, $this->plugin_main_file ) = explode( '/', $this->plugin_basename );
		$this->plugin_url = plugin_dir_url( $this->plugin_basename );
		$this->plugin_path = trailingslashit( dirname( __FILE__ ) );
		$this->lib_path = $this->plugin_path . '/includes/libraries';
		$this->assets_url = $this->plugin_url . 'admin/assets/';

		// Load after WooCommerce
		add_action( 'woocommerce_init', array( $this, 'init' ), 20 );

		spl_autoload_register( array( $this, 'autoload' ) );
	}



	/**
	 * Initiate
	 */
	public function init()
	{
		// Set up localisation
		$this->load_plugin_textdomain();

		$this->includes();

		AW_Post_Types::init();
		AW_Cron::init();
		AW_Ajax::init();

		$this->cache = new AW_Cache_Helper();
		$this->email = new AW_Email();
		$this->wishlist = new AW_Wishlist();
		$this->session_tracker = new AW_Session_Tracker();
		$this->abandoned_cart = new AW_Abandoned_Cart_Manager();
		$this->integrations = new AW_Integrations();
		$this->options = new AW_Options();
		$this->queue_manager = new AW_Queue_Manager();
		$this->conversion_manager = new AW_Conversion_Manager();
		$this->licence = new AW_Licence();
		$this->language_helper = new AW_Language_Helper();

		do_action( 'automatewoo_load_addons' );

		// Load and init the triggers
		// Actions don't load until required by admin interface or when a workflow runs
		$this->load_triggers();
		$this->init_triggers();

		if ( is_admin() )
		{
			$this->admin = new AW_Admin();
			$this->admin_helper = new AW_Admin_Helper();
			$this->system_checker = new AW_System_Checker();
			AW_Updater::init();
			AW_Install::init();
		}

		// TODO improve this debug system
		if ( WP_DEBUG )
		{
			add_action('wp_loaded', array( $this, 'enable_debug' ) );
		}

		do_action( 'automatewoo_init' );
		do_action( 'automatewoo_loaded' );
	}


	/**
	 * Autoload any class that isn't a trigger or an action
	 *
	 * @since 2.0
	 *
	 * @param $class
	 *
	 * @return mixed|void
	 */
	public function autoload( $class )
	{
		if ( substr( $class, 0, 3 ) != 'AW_')
			return false;

		$file = str_replace( 'AW_', '/', $class );
		$file = str_replace( '_', '-', $file );
		$file = strtolower( $file );

		$abstracts = array( '/action', '/trigger', '/field', '/query', '/model', '/query-custom-table', '/system-check', '/integration' );

		if ( in_array( $file, $abstracts ) )
		{
			include dirname(__FILE__) . '/includes/abstracts' . $file . '.php';
		}
		elseif ( strstr( $file, '/admin-' ) )
		{
			$file = str_replace( '/admin-', '/admin/', $file );

			include dirname(__FILE__) . $file . '.php';
		}
		else
		{
			$file = str_replace( '/trigger-', 'triggers/', $file );
			$file = str_replace( '/action-', 'actions/', $file );
			$file = str_replace( '/field-', 'fields/field-', $file );
			$file = str_replace( '/query-', 'queries/query-', $file );
			$file = str_replace( '/model-', 'models/model-', $file );
			$file = str_replace( '/system-check-', 'system-checks/system-check-', $file );
			$file = str_replace( '/integration-', 'integrations/integration-', $file );

			include dirname(__FILE__) . '/includes/' . $file . '.php';
		}
	}



	/**
	 * Includes
	 */
	public function includes()
	{
		include_once( 'includes/compatibility.php' );
		include_once( 'includes/helpers.php' );

		include_once( $this->lib_path . '/easy-user-tags/easy-user-tags.php' );

		// Admin
		if ( is_admin() )
		{
			include_once( 'admin/admin.php' );
			include_once( 'admin/admin-helper.php' );
		}
	}



	/**
	 * Load triggers
	 *
	 * @since 2.0.0
	 */
	public function load_triggers()
	{
		if ( did_action('automatewoo_triggers_loaded') ) return;

		do_action('automatewoo_before_triggers_loaded');

		new AW_Trigger_Order_Status_Changes();
		new AW_Trigger_Order_Processing();
		new AW_Trigger_Order_Completed();
		new AW_Trigger_Order_Cancelled();
		new AW_Trigger_Order_On_Hold();
		new AW_Trigger_Order_Refunded();
		new AW_Trigger_Order_Pending();
		new AW_Trigger_Order_Placed();
		new AW_Trigger_Order_Note_Added();

		new AW_Trigger_User_Absent();
		new AW_Trigger_User_New_Account();
		new AW_Trigger_User_Total_Spend_Reaches();
		new AW_Trigger_User_Order_Count_Reaches();
		new AW_Trigger_User_Leaves_Review();
		new AW_Trigger_Guest_Leaves_Review();
		new AW_Trigger_User_Purchases_From_Category();
		new AW_Trigger_User_Purchases_From_Tag();
		new AW_Trigger_User_Purchases_From_Taxonomy_Term();
		new AW_Trigger_User_Purchases_Specific_Product();
		new AW_Trigger_User_Purchases_Product_Variation_With_Attribute();

		new AW_Trigger_Abandoned_Cart_User();
		new AW_Trigger_Abandoned_Cart_Guest();

		if ( AutomateWoo()->integrations->subscriptions_enabled() )
		{
			new AW_Trigger_Subscription_Status_Changed();
			new AW_Trigger_Subscription_Payment_Complete();
			new AW_Trigger_Subscription_Payment_Failed();
			new AW_Trigger_Subscription_Trial_End();
		}

		new AW_Trigger_Wishlist_Item_Goes_On_Sale();
		new AW_Trigger_Wishlist_Reminder();
		new AW_Trigger_Wishlist_Item_Added();

		new AW_Trigger_Workflow_Times_Run_Reaches();

		do_action('automatewoo_triggers_loaded');
	}


	/**
	 * Loads triggers.
	 *
	 * Triggers must always be loaded on init so that their hooks can be added.
	 */
	public function init_triggers()
	{
		if ( did_action('automatewoo_init_triggers') ) return;
		do_action('automatewoo_init_triggers');
	}


	/**
	 * Load actions
	 *
	 * @since 2.0
	 */
	public function load_actions()
	{
		if ( did_action('automatewoo_actions_loaded') ) return;

		do_action('automatewoo_before_actions_loaded');

		new AW_Action_Send_Email();

		if ( AutomateWoo()->options->twilio_integration_enabled )
		{
			new AW_Action_Send_SMS_Twilio();
		}

		new AW_Action_Change_User_Type();
		new AW_Action_Update_User_Meta();
		new AW_Action_User_Add_Tags();
		new AW_Action_User_Remove_Tags();

		new AW_Action_Change_Order_Status();
		new AW_Action_Update_Order_Meta();
		new AW_Action_Resend_Order_Email();
		new AW_Action_Trigger_Order_Action();

		if ( AutomateWoo()->options->active_campaign_integration_enabled )
		{
			new AW_Action_Active_Campaign_Add_User_To_List();
			new AW_Action_Active_Campaign_Add_Tag();
			new AW_Action_Active_Campaign_Remove_Tag();
		}

		if ( AutomateWoo()->options->mailchimp_integration_enabled )
		{
			new AW_Action_MailChimp_Subscribe();
			new AW_Action_MailChimp_Unsubscribe();
		}

		new AW_Action_Add_To_Campaign_Monitor();
		new AW_Action_Add_To_Mail_Chimp_DEPRECATED();
		new AW_Action_Add_To_Mad_Mimi_List();

		if ( AutomateWoo()->integrations->subscriptions_enabled() )
		{
			new AW_Action_Change_Subscription_Status();
		}

		new AW_Action_Clear_Queued_Events();
		new AW_Action_Custom_Function();
		new AW_Action_Update_Product_Meta();
		new AW_Action_Change_Post_Status();

		do_action('automatewoo_actions_loaded');
	}


	/**
	 * Init actions. Also calls load just in case.
	 *
	 * Actions need only be loaded when they are needed.
	 * This method can be called multiple times.
	 */
	public function init_actions()
	{
		if ( did_action('automatewoo_init_actions') ) return;
		$this->load_actions();
		do_action('automatewoo_init_actions');
	}


	/**
	 * @param $trigger_name string
	 *
	 * @return AW_Trigger/false
	 */
	public function get_registered_trigger( $trigger_name )
	{
		if ( ! isset( $this->registered_triggers[$trigger_name] ) )
			return false;

		return $this->registered_triggers[$trigger_name];
	}


	/**
	 * @param $action_name string
	 *
	 * @return AW_Action/false
	 */
	public function get_registered_action( $action_name )
	{
		$this->init_actions();

		if ( ! isset( $this->registered_actions[$action_name] ) )
			return false;

		return $this->registered_actions[$action_name];
	}


	/**
	 * @return array
	 */
	public function get_actions()
	{
		$this->init_actions();
		return $this->registered_actions;
	}



	/**
	 *
	 */
	public function load_plugin_textdomain()
	{
		load_plugin_textdomain( 'automatewoo', false, "automatewoo/languages" );
	}



	/**
	 *
	 */
	public function enable_debug()
	{
		$this->debug = true;

		if ( aw_request('aw-debug-cron-15') )
		{
			echo "AutomateWoo Fifteen Minute Worker";
			do_action('automatewoo_fifteen_minute_worker');
			exit();
		}

		if ( aw_request('aw-debug-cron-30') )
		{
			echo "AutomateWoo Thirty Minute Worker";
			do_action('automatewoo_thirty_minute_worker');
			exit();
		}

		if ( aw_request('aw-debug-cron-hourly') )
		{
			echo "AutomateWoo Hourly Worker";
			do_action('automatewoo_hourly_worker');
			exit();
		}

		if ( aw_request('aw-debug-cron-daily') )
		{
			echo "AutomateWoo Daily Worker";
			do_action('automatewoo_daily_worker');
			exit();
		}


		if ( aw_request('aw-debug-cron-four-hourly') )
		{
			echo "AutomateWoo Four Hourly Worker";
			do_action('automatewoo_four_hourly_worker');
			exit();
		}
	}


	/**
	 * What type of request is this?
	 * string $type ajax, frontend or admin.
	 *
	 * @return bool
	 */
	public function is_request( $type )
	{
		switch ( $type )
		{
			case 'admin' :
				return is_admin();
			case 'ajax' :
				return defined( 'DOING_AJAX' );
			case 'cron' :
				return defined( 'DOING_CRON' );
			case 'frontend' :
				return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
		}
	}



	/**
	 * @return AutomateWoo - Main instance
	 */
	public static function instance()
	{
		if ( is_null( self::$_instance ) )
		{
			self::$_instance = new self();
		}
		return self::$_instance;
	}


}



/**
 * Returns the main instance
 */
function AutomateWoo()
{
	return AutomateWoo::instance();
}
AutomateWoo();


endif; // class_exists check
