<?php
/**
 * @class       AW_Abandoned_Cart_Manager
 * @package     AutomateWoo
 */

class AW_Abandoned_Cart_Manager
{
	/**
	 * Constructor
	 */
	public function __construct()
	{
		add_action( 'automatewoo_two_days_worker', array( $this, 'clean_stored_carts' ) );
		add_action( 'wp_loaded', array( $this, 'catch_restore_cart_link' ) );

		// bail if no workflows set up
		if ( ! $this->in_use() ) return;

		add_action( 'woocommerce_add_to_cart', array( $this, 'maybe_store_cart' ) );
		add_action( 'woocommerce_cart_item_removed', array( $this, 'maybe_store_cart' ) );
		add_action( 'woocommerce_cart_item_restored', array( $this, 'maybe_store_cart' ) );
		add_action( 'woocommerce_after_cart_item_quantity_update', array( $this, 'maybe_store_cart' ) );

		add_action( 'wp_login', array( $this, 'wp_login' ), 10, 2 );

		add_action( 'woocommerce_cart_emptied', array( $this, 'cart_emptied' ) );
		add_action( 'woocommerce_checkout_order_processed', array( $this, 'empty_after_order_created' ) );

		add_action( 'wp_footer', array( $this, 'js' ) );
	}


	/**
	 * Logic to determine whether we should save the cart on certain hooks
	 */
	public function maybe_store_cart()
	{
		if ( $user_id = AutomateWoo()->session_tracker->get_detected_user_id() )
		{
			$this->store_user_cart( $user_id );
		}
		elseif ( $guest = AutomateWoo()->session_tracker->get_current_guest() )
		{
			// Store a guest cart if the guest has been stored in the database
			$this->store_guest_cart( $guest );

			$guest->update_last_active();
		}
	}


	/**
	 * Clear the stored guest before their cookie key is updated.
	 * Store the cart upon login
	 */
	public function wp_login( $user_login, $user )
	{
		$guest = new AW_Model_Guest();
		$guest->get_by( 'tracking_key', AutomateWoo()->session_tracker->get_tracking_key() );

		$guest->delete_cart(); // clear guest cart if it exists

		$guest->delete();

		// create new cart associated with the user
		$this->store_user_cart( $user->ID );
	}




	/**
	 * Attempts to update or insert carts for guests
	 *
	 * @param AW_Model_Guest $guest
	 *
	 * @return bool
	 */
	public function store_guest_cart( $guest )
	{
		if ( ! $guest )
			return false;

		$cart = $guest->get_cart();

		if ( $cart )
		{
			if ( 0 === sizeof( WC()->cart->get_cart() ) )
				$cart->delete();
			else
				$cart->sync();
		}
		else
		{
			// create new cart
			$ac = new AW_Model_Abandoned_Cart();
			$ac->guest_id = $guest->id;
			$ac->sync();
		}

		return true;
	}


	/**
	 * Attempts to store cart for a registered user whether they are logged in or not
	 *
	 * Won't store empty carts.
	 *
	 * @param bool $user_id
	 *
	 * @return bool|int
	 */
	public function store_user_cart( $user_id = false )
	{
		if ( ! $user_id )
		{
			// get user
			if ( ! $user_id = AutomateWoo()->session_tracker->get_detected_user_id() )
				return false;
		}

		// does this user already have a stored cart?
		$existing_ac = new AW_Model_Abandoned_Cart();
		$existing_ac->get_by( 'user_id', $user_id );

		// if cart already exists
		if ( $existing_ac->exists )
		{
			// delete cart if empty otherwise update it
			if ( 0 === sizeof( WC()->cart->get_cart() ) )
				$existing_ac->delete();
			else
				$existing_ac->sync();

			return true;
		}
		else
		{
			// if the cart doesn't already exist
			// and there are no items in cart no there is no need to insert
			if ( 0 === sizeof( WC()->cart->get_cart() ) )
				return false;

			// create a new stored cart for the user
			$ac = new AW_Model_Abandoned_Cart();
			$ac->user_id = $user_id;
			$ac->sync();

			return true;
		}

	}


	/**
	 * This event will fire when an order is placed and the cart is emptied NOT when a user empties their cart.
	 */
	public function cart_emptied()
	{
		$guest = AutomateWoo()->session_tracker->get_current_guest();
		$user_id = AutomateWoo()->session_tracker->get_detected_user_id();

		if ( $user_id )
		{
			$cart = new AW_Model_Abandoned_Cart();
			$cart->get_by( 'user_id', $user_id );
			$cart->delete();
		}

		if ( $guest )
		{
			// Ensure carts are cleared for users and guests registered at checkout
			$guest->delete_cart();
		}
	}


	/**
	 * Ensure the stored abandoned cart is removed when an order is created
	 *
	 * Clears even if payment has not gone through
	 *
	 * @param $order_id
	 */
	public function empty_after_order_created( $order_id )
	{
		$order = wc_get_order( $order_id );

		$user_id = $order->get_user_id();

		if ( $user_id > 0 )
		{
			$cart = new AW_Model_Abandoned_Cart();
			$cart->get_by( 'user_id', $user_id );
			$cart->delete();
		}

		// order placed by a guest or by a guest that signed up at checkout

		$guest = new AW_Model_Guest();
		$guest->get_by( 'email', $order->billing_email );
		$guest->delete_cart();

		// Delete cart by guest cookie key
		if ( $guest = AutomateWoo()->session_tracker->get_current_guest() )
		{
			$guest->delete_cart();
		}
	}



	/**
	 * Add ajax email capture to checkout
	 */
	public function js()
	{
		switch( AutomateWoo()->options->guest_email_capture_scope )
		{
			case 'none': return; break;
			case 'checkout':
				if ( ! is_checkout() ) return;
				break;
		}

		?>
			<script type="text/javascript">
				(function($){

					var email = '';

					$(document).on('blur', '#billing_email, .automatewoo-capture-guest-email', function(){

						// hasn't changed
						if ( email == $(this).val() ) {
							return;
						}

						email = $(this).val();

						$.ajax({
							method: "POST",
							url: woocommerce_params.ajax_url,
							cache: false,
							data: {
								action: 'aw_capture_email',
								email: email
								<?php if ( AutomateWoo()->integrations->is_wpml() ): ?>
									,language: '<?php echo esc_js( wpml_get_current_language() ) ?>'
								<?php endif; ?>
							}
						});

					});

				})(jQuery);
			</script>
		<?php
	}



	/**
	 * Restores a cart when URL is clicked
	 */
	public function catch_restore_cart_link()
	{
		$cart_id = absint( aw_request('aw-restore-cart') );

		if ( ! $cart_id )
			return;

		$cart = new AW_Model_Abandoned_Cart( $cart_id );

		if ( ! $cart->exists || ! is_array( $cart->items ) )
			return;

		// merge restored items with existing
		$existing_items = WC()->cart->get_cart_for_session();

		foreach ( $cart->items as $item_key => $item )
		{
			// item already exists in cart
			if ( isset( $existing_items[$item_key] ) )
				continue;

			WC()->cart->add_to_cart( $item['product_id'], $item['quantity'], $item['variation_id'], $item['variation']  );
		}
	}



	/**
	 * Delete stored carts older than 45 days
	 */
	public function clean_stored_carts()
	{
		global $wpdb;

		$delay_date = new DateTime(); // UTC
		$delay_date->setTimestamp( current_time('timestamp', true ) );
		$delay_date->modify("-45 days");

		$wpdb->query( $wpdb->prepare("
			DELETE FROM ". $wpdb->prefix . AutomateWoo()->table_name_abandoned_cart . "
			WHERE last_modified < %s",
			$delay_date->format('Y-m-d H:i:s')
		));
	}


	/**
	 * Checks if abandoned cart triggers are being used
	 */
	public function in_use()
	{
		$cache_key = 'abandoned_cart_in_use';
		$in_use = AutomateWoo()->cache->get( $cache_key );

		if ( ! $in_use )
		{
			$workflows = get_posts(array(
				'post_type' => AutomateWoo()->post_type,
				'fields' => 'ids',
				'posts_per_page' => 1,
				'meta_query' => array(
					array(
						'key' => 'trigger_name',
						'value' => array(
							'guest_abandoned_cart',
							'abandoned_cart'
						),
					)
				)
			));

			$in_use = empty( $workflows ) ? 'no': 'yes';

			AutomateWoo()->cache->set( $cache_key, $in_use );
		}

		return $in_use === 'yes';
	}


}
