<?php
/**
 * @class       AW_Trigger
 * @package     AutomateWoo/Abstracts
 * @since       1.0.0
 */

abstract class AW_Trigger
{
	/** @var string */
	public $title;

	/** @var string */
	public $name;

	/** @var string */
	public $description;

	/** @var string */
	public $group = 'Other';

	/** @var bool */
	public $allow_queueing = true;

	/** @var array */
	public $fields = array();

	/** @var array */
	public $workflows = array();

	/** @var bool */
	public $workflows_loaded;

	/** @var array */
	public $options;

	/** @var bool */
	protected $_fields_loaded = false;


	abstract public function register_hooks();

	abstract public function load_fields();



	/**
	 * Construct
	 */
	public function __construct()
	{
		add_action( 'automatewoo_init_triggers', array( $this, 'init' ) );
	}



	/**
	 * Construct
	 */
	public function init()
	{
		$this->register_hooks();

		// Register the class
		AutomateWoo()->registered_triggers[$this->name] = $this;
	}



	/**
	 * @param $option object
	 */
	public function add_field( $option )
	{
		$option->set_name_base( 'aw_workflow_data[trigger_options]' );
		$this->fields[ $option->get_name() ] = $option;
	}


	/**
	 * @param $option_name
	 */
	public function remove_field( $option_name )
	{
		if ( isset( $this->fields[ $option_name ] ) )
			unset( $this->fields[ $option_name ] );
	}


	/**
	 * @param $name
	 *
	 * @return mixed
	 */
	public function get_field( $name )
	{
		if ( ! $this->_fields_loaded )
		{
			$this->load_fields();
			$this->_fields_loaded = true;
		}

		if ( ! isset( $this->fields[$name] ) )
			return false;

		return $this->fields[$name];
	}


	/**
	 * @return array
	 */
	public function get_fields()
	{
		if ( ! $this->_fields_loaded )
		{
			$this->load_fields();
			$this->_fields_loaded = true;
		}

		return $this->fields;
	}



	public function has_workflows()
	{
		$workflows = get_posts(array(
			'post_type' => AutomateWoo()->post_type,
			'fields' => 'ids',
			'posts_per_page' => 1,
			'meta_query' => array(
				array(
					'key' => 'trigger_name',
					'value' => $this->name
				)
			)
		));

		return ! empty( $workflows );
	}


	/**
	 *
	 */
	public function get_workflows()
	{
		if ( ! $this->workflows_loaded )
		{
			$query = new AW_Query_Workflows();
			$query->set_trigger( $this );
			$this->workflows = $query->get_results();
		}

		return $this->workflows;
	}


	/**
	 * Every data item registered with the trigger should be supplied to this method in its object form.
	 * E.g. a 'user' should be passed as a WP_User object, and an 'order' should be passed as a WC_Order object
	 *
	 * @param array $data_items
	 */
	public function maybe_run( $data_items = array() )
	{
		// Get all workflows that are registered to use this trigger
		$workflows = $this->get_workflows();

		if ( ! $workflows )
			return;

		// Check if each workflow should be run based on its options
		foreach ( $workflows as $workflow )
		{
			$workflow->set_data_items( $data_items );
			$trigger = $workflow->get_trigger();

			if ( $trigger )
			{
				// determine if a workflows should run
				if ( $this->validate_workflow_language( $workflow ) && $this->validate_workflow( $workflow ) )
				{
					// hook in, return false to prevent run
					if ( apply_filters( 'automatewoo_custom_validate_workflow', true, $workflow ) )
					{
						$workflow->run();
					}
				}
			}
		}
	}


	/**
	 * @return string
	 */
	public function get_name()
	{
		return $this->name;
	}


	/**
	 * @return string
	 */
	public function get_title()
	{
		return $this->title;
	}

	/**
	 * @return string|null
	 */
	public function get_description()
	{
		return $this->description;
	}

	/**
	 * @return string
	 */
	public function get_description_html()
	{
		if ( ! $this->get_description() )
			return '';

		return '<p class="aw-field-description">' . $this->get_description() .'</p>';
	}


	/**
	 *
	 */
	public function set_options($options)
	{
		$this->options = $options;
	}


	/**
	 * Will return all data if $field is false
	 *
	 * @param string $field
	 * @return mixed
	 */
	public function get_option( $field )
	{
		if ( ! $field ) return false;

		$value = false;

		if ( isset( $this->options[$field] ) )
		{
			$value = $this->options[$field];
		}

		return apply_filters( 'automatewoo_trigger_option', $value, $field, $this );
	}


	/**
	 * This method is called just before a queued workflow runs
	 *
	 * @param AW_Model_Workflow $workflow
	 *
	 * @return bool
	 */
	public function validate_before_queued_event( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');

		// Important: Not all triggers will have a user data item
		if ( $user )
		{
			// if times run field is present validate here as its a common field
			if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
				return false;

			// validate user type if its present
			if ( ! $this->validate_user_type_field( $trigger, $user ) )
				return false;

			// validate user tags
			if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
				return false;
		}

		return true;
	}



	/**
	 * Checks if this trigger's language matches that of the user or guest
	 *
	 * @param AW_Model_Workflow $workflow
	 *
	 * @return bool
	 */
	public function validate_workflow_language($workflow )
	{
		if ( ! AutomateWoo()->integrations->is_wpml() ) return true;

		$language = $workflow->get_language();

		if ( ! $language ) return true; // workflow has no set language

		if ( $user = $workflow->get_data_item('user') )
		{
			return AutomateWoo()->language_helper->get_user_language( $user ) == $language;
		}
		elseif ( $guest = $workflow->get_data_item('guest') )
		{
			return AutomateWoo()->language_helper->get_guest_language( $guest ) == $language;
		}
		else
		{
			return true;
		}
	}


	/**
	 *
	 */
	protected function add_user_limit_field()
	{
		$limit = new AW_Field_Number_Input();
		$limit->set_name('once_only');
		$limit->set_title( __( 'Limit Per User', 'automatewoo' ));
		$limit->set_description( __( 'If checked the workflow will run a set amount of times for each user.', 'automatewoo'  ) );
		$limit->set_placeholder( __( 'Leave blank for no limit', 'automatewoo'  ) );
		$this->add_field($limit);
	}


	/**
	 *
	 */
	protected function add_guest_limit_field()
	{
		$limit = new AW_Field_Number_Input();
		$limit->set_name('limit_per_guest');
		$limit->set_title( __( 'Limit Per Guest', 'automatewoo' ));
		$limit->set_description( __( 'If checked the workflow will run a set amount of times for each guest.', 'automatewoo'  ) );
		$limit->set_placeholder( __( 'Leave blank for no limit', 'automatewoo'  ) );
		$this->add_field($limit);
	}



	/**
	 *
	 */
	protected function add_user_tags_field()
	{
		$user_has_tags = new AW_Field_User_Tags();
		$user_has_tags->set_title( __('User Has Tags', 'automatewoo' ) );

		if ( $this->group == 'Order')
		{
			$user_has_tags->set_description(__( 'Only trigger when the user has all of the selected tags. Note this will always fail for guest orders because guests can have no tags.', 'automatewoo'  ) );
		}
		else
		{
			$user_has_tags->set_description(__('Only trigger when the user has all of the selected tags.', 'automatewoo'  ) );
		}


		$user_missing_tags = new AW_Field_User_Tags();
		$user_missing_tags->set_name('user_missing_tags');
		$user_missing_tags->set_title( __("User Missing Tags", 'automatewoo' ) );

		if ( $this->group == 'Order')
		{
			$user_missing_tags->set_description(__( "Only trigger when the user is missing all of the selected tags. Note this will always fail for guest orders because guests can have no tags.", 'automatewoo'  ) );
		}
		else
		{
			$user_missing_tags->set_description(__("Only trigger when the user is missing all of the selected tags.", 'automatewoo'  ) );
		}

		$this->add_field($user_has_tags);
		$this->add_field($user_missing_tags);
	}



	protected function add_field_validate_queued_order_status()
	{
		$field = new AW_Field_Checkbox();
		$field->set_name('validate_order_status_before_queued_run');
		$field->set_title( __('Recheck Status Before Run', 'automatewoo' ) );
		$field->default_to_checked = true;
		$field->set_description(
			__( "This is useful for Workflows that are not run immediately as it ensures the status of the order hasn't changed since initial trigger." ,
				'automatewoo'  ) );

		$this->add_field( $field );
	}



	protected function add_field_user_pause_period()
	{
		$field = new AW_Field_Number_Input();
		$field->set_name( 'user_pause_period' );
		$field->set_title( __( 'User Pause Period (days)', 'automatewoo' ) );
		$field->set_description( __( 'Can be used to ensure that this trigger will only send once in a set period to a user or guest.', 'automatewoo' ) );
		$this->add_field( $field );
	}


	/**
	 * Checks if a user has all of the tags
	 *
	 * @param $user WP_User
	 * @param $trigger AW_Trigger
	 *
	 * @return bool
	 */
	protected function validate_user_tag_fields( $user, $trigger )
	{
		$has_tags = $trigger->get_option('user_tags');
		$missing_tags = $trigger->get_option('user_missing_tags');

		if ( ! $user ) return false;

		if ( is_array($has_tags) || is_array($missing_tags) )
		{
			// always fail for guest orders
			if ( ! $user->ID === 0 ) return false;
		}


		if ( is_array($has_tags) )
		{
			foreach( $has_tags as $tag )
			{
				if ( ! is_object_in_term( $user->ID, 'user_tag', $tag ) )
					return false;
			}
		}

		if ( is_array($missing_tags) )
		{
			foreach( $missing_tags as $tag )
			{
				if ( is_object_in_term( $user->ID, 'user_tag', $tag ) )
					return false;
			}
		}


		return true;
	}



	/**
	 * Order status field must be named 'order_status'
	 *
	 * @param $trigger AW_Trigger
	 * @param $order WC_Order
	 *
	 * @return bool
	 * @since 2.0
	 */
	protected function validate_order_status_field( $trigger, $order )
	{
		$status = $trigger->get_option('order_status');

		if ( ! $status ) return true;

		$status = 'wc-' === substr( $status, 0, 3 ) ? substr( $status, 3 ) : $status;

		// wrong status
		if ( $order->get_status() !== $status )
			return false;

		return true;
	}


	/**
	 * @param $trigger AW_Trigger
	 * @param $user WP_User
	 *
	 * @return bool
	 *
	 * @since 2.0
	 */
	protected function validate_user_type_field( $trigger, $user )
	{
		$required_user_type = $trigger->get_option('user_type');

		if ( ! $required_user_type ) return true; // no user type requirement

		if ( ! $user ) return false; // user missing

		if ( $user->ID === 0 ) // order placed by guest
		{
			if ( $required_user_type !== 'guest' )
				return false;
		}
		else // order placed by registered user
		{
			// Validate User type
			if ( ! user_can( $user->ID, $required_user_type ) )
				return false;
		}

		return true;
	}


	/**
	 * Note: Has support for orders placed by guests
	 *
	 * @param $workflow AW_Model_Workflow
	 * @param $trigger AW_Trigger
	 * @param $user WP_User
	 *
	 * @return bool
	 *
	 * @since 2.0
	 */
	protected function validate_limit_per_user( $workflow, $trigger, $user )
	{
		$limit = $trigger->get_option('once_only');

		if ( ! $limit  ) return true;

		$times_run = $workflow->get_times_run_for_user( $user );

		if ( $limit <= $times_run )
			return false;

		return true;
	}



	/**
	 * @param $workflow AW_Model_Workflow
	 * @param $trigger AW_Trigger
	 * @param $guest AW_Model_Guest
	 *
	 * @return bool
	 *
	 * @since 2.0
	 */
	protected function validate_limit_per_guest( $workflow, $trigger, $guest )
	{
		$limit = $trigger->get_option('limit_per_guest');

		if ( ! $limit  ) return true;

		$times_run = $workflow->get_times_run_for_guest( $guest );

		if ( $limit <= $times_run )
			return false;

		return true;
	}


	/**
	 * Return false if field exists and is not the first order. Otherwise returns true
	 *
	 * @param $workflow AW_Model_Workflow
	 * @param $trigger AW_Trigger
	 *
	 * @return bool
	 *
	 * @since 2.1.0
	 */
	protected function validate_is_first_order_field( $workflow, $trigger )
	{
		$is_users_first_order = $trigger->get_option('is_users_first_order');

		// option not checked
		if ( ! $is_users_first_order ) return true;

		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order ) return true; // bail

		$order_query_args = array(
			'posts_per_page' => 1,
			'post_type' => 'shop_order',
			'post_status' => array( 'wc-processing', 'wc-completed', 'wc-pending' ),
			'fields' => 'ids',
			'post__not_in' => array( $order->id ),
			'meta_query' => array()
		);

		if ( $user->ID === 0 ) // order placed by guest
		{
			$order_query_args['meta_query'] = array(
				array(
					'key' => '_billing_email',
					'value' => $user->user_email,
				)
			);
		}
		else
		{
			$order_query_args['meta_query'][] = array(
				'key' => '_customer_user',
				'value' => $user->ID,
			);
		}

		$orders = get_posts( $order_query_args );

		return empty( $orders );
	}


	/**
	 * Also works for guests
	 *
	 * @param $workflow AW_Model_Workflow
	 * @return bool
	 */
	protected function validate_field_user_pause_period( $workflow )
	{
		$trigger = $workflow->get_trigger();

		$period = $trigger->get_option('user_pause_period');
		$user = $workflow->get_data_item('user');
		$guest = $workflow->get_data_item('guest');

		if ( empty( $period ) ) return true; // no pause period set

		if ( ! $user && ! $guest ) return true; // must have a user or guest

		$hours = $period * 24;

		$period_date = new DateTime( current_time('mysql', true ) ); // GMT
		$period_date->modify("-$hours hours");

		// Check to see if this workflow has run since the period date
		$log_query = new AW_Query_Logs();
		$log_query->where( 'workflow_id', $workflow->id );
		$log_query->set_limit(1);
		$log_query->where('date', $period_date->format('Y-m-d H:i:s'), '>');

		if ( $user )
		{
			if ( $user->ID === 0 ) // guest user
			{
				$log_query->where( 'guest_email', $user->user_email );
			}
			else
			{
				$log_query->where( 'user_id', $user->ID );
			}
		}
		elseif( $guest )
		{
			$log_query->where( 'guest_email', $guest->email );
		}

		if ( $log_query->get_results() )
			return false;

		return true;
	}




	/**
	 * @param $allowed_statuses array|string
	 * @param $current_status string
	 *
	 * @return bool
	 */
	protected function validate_status_field( $allowed_statuses, $current_status )
	{
		// allow all if left blank
		if ( empty( $allowed_statuses ) ) return true;

		if ( is_array( $allowed_statuses ) )
		{
			// multi status match
			$with_prefix_match = in_array( 'wc-' . $current_status, $allowed_statuses );
			$no_prefix_match = in_array( $current_status, $allowed_statuses );

			// at least one has to match
			if ( ! $with_prefix_match && ! $no_prefix_match )
				return false;
		}
		else
		{
			// single status match, remove prefix
			$allowed_statuses = 'wc-' === substr( $allowed_statuses, 0, 3 ) ? substr( $allowed_statuses, 3 ) : $allowed_statuses;

			if ( $allowed_statuses != $current_status )
				return false;
		}

		return true;
	}

}