<?php
/**
 * @class       AW_Conversion_Manager
 * @since       2.1.0
 * @package     AutomateWoo
 */

class AW_Conversion_Manager
{
	/**
	 * Max number of days for a purchase to be considered a conversion
	 * @var int
	 */
	public $conversion_window;


	/**
	 * Constructor
	 */
	public function __construct()
	{
		$this->conversion_window = apply_filters( 'automatewoo_conversion_window', AutomateWoo()->options->conversion_window );

		add_action( 'woocommerce_checkout_order_processed', array( $this, 'check_order' ) );
	}


	/**
	 * @param $order_id
	 * @return void
	 */
	public function check_order( $order_id )
	{
		$order = wc_get_order( $order_id );

		if ( ! $order ) return;

		$user = $order->get_user();


		if ( $user )
		{
			$logs_by_user_id = $this->get_logs_by_user_id( $user->ID );

			// User may have just created an account
			$logs_by_email = $this->get_logs_by_email( $user->user_email );
		}
		else
		{
			// guest order
			$logs_by_email = $this->get_logs_by_email( $order->billing_email );

			$logs_by_user_id = array();
		}

		$logs = array_merge( $logs_by_email, $logs_by_user_id );

		// check that at least one logs shows that it has been opened i.e. has tracking data
		if ( $logs )
		{
			/** @var $log AW_Model_Log  */
			foreach ( $logs as $log )
			{
				if ( $log->get_meta('tracking_data') )
				{
					// mark the order as a conversion
					update_post_meta( $order_id, '_aw_conversion', $log->workflow_id );
					update_post_meta( $order_id, '_aw_conversion_log', $log->id );

					do_action( 'automatewoo_abandoned_cart_conversion', $order, $log );

					break;
				}
			}

		}
	}


	/**
	 * @return DateTime
	 */
	public function get_conversion_window_date()
	{
		$date = new DateTime(); // UTC
		$date->setTimestamp( current_time('timestamp', true )  );
		$date->modify("-$this->conversion_window days");
		return $date;
	}



	/**
	 * @param $email
	 *
	 * @return array
	 */
	public function get_logs_by_email( $email )
	{
		$query = new AW_Query_Logs();
		$query->set_ordering('date', 'DESC');
		$query->where( 'conversion_tracking_enabled', true );
		$query->where( 'date', $this->get_conversion_window_date(), '>' );

		// Check if the user exists and they just haven't signed in
		if ( $user = get_user_by( 'email', $email ) )
		{
			$query->where( 'user_id', $user->ID );
		}
		else
		{
			$query->where( 'guest_email', $email );
		}

		$logs = $query->get_results();

		if ( ! $logs ) $logs = array();

		return $logs;
	}


	/**
	 * @param $id
	 *
	 * @return array
	 */
	public function get_logs_by_user_id( $id )
	{
		$query = new AW_Query_Logs();
		$query->set_ordering('date', 'DESC');
		$query->where( 'user_id', $id );
		$query->where( 'conversion_tracking_enabled', true );
		$query->where( 'date', $this->get_conversion_window_date(), '>' );
		$logs = $query->get_results();

		if ( ! $logs ) $logs = array();

		return $logs;
	}

}
