<?php
/**
 * Cron / Worker manager
 *
 * @class 		AW_Cron
 * @package		AutomateWoo
 */

class AW_Cron
{
	/**
	 * @var array
	 */
	static $workers = array(
		'fifteen_minute', 'thirty_minute', 'hourly', 'four_hourly', 'daily', 'two_days', 'weekly'
	);


	/**
	 * Init cron
	 */
	static function init()
	{
		add_filter( 'cron_schedules', array( __CLASS__, 'add_schedules' ) );

		foreach ( self::$workers as $worker )
		{
			add_action( 'automatewoo_' . $worker . '_worker', array( __CLASS__, 'before_worker' ), 1 );
		}

		add_action('admin_init', array( __CLASS__, 'add_events' ) );
//		add_action('admin_init', array( __CLASS__, 'clean_up' ) );
	}


	/**
	 * Prevents workers from working if they have done so in the past 6 minutes
	 *
	 * @since 2.1.5
	 */
	static function before_worker()
	{
		if ( self::is_worker_locked( current_action() ) )
		{
			// block action
			remove_all_actions( current_action() );
		}
		else
		{
			@set_time_limit(300); // 5 min

			// allow action
			self::update_worker_lock( current_action() );
		}
	}


	/**
	 * @since 2.1.5
	 *
	 * @return string
	 */
	static function get_worker_lock()
	{
		$worker_lock = get_option( 'automatewoo_worker_lock' );
		return is_array( $worker_lock ) ? $worker_lock : array();
	}


	/**
	 * Checks if workers started running less than 6 minutes ago
	 *
	 * @since 2.1.5
	 *
	 * @param $action
	 *
	 * @return bool
	 */
	static function is_worker_locked( $action )
	{
		if ( AutomateWoo()->debug ) return false;

		$worker_lock = self::get_worker_lock();

		if ( empty( $worker_lock[$action] ) ) return false;

		$time_last_run = $worker_lock[$action];

		$time_unblocked = new DateTime( current_time( 'mysql', true ) );
		$time_unblocked->modify( '-6 minutes' );

		return $time_last_run > $time_unblocked->getTimestamp();
	}


	/**
	 * @since 2.1.5
	 *
	 * @param $action
	 */
	static function update_worker_lock( $action )
	{
		$worker_lock = self::get_worker_lock();

		$worker_lock[$action] = current_time( 'timestamp', true );

		update_option( 'automatewoo_worker_lock', $worker_lock, false );
	}


	/**
	 * Add cron workers
	 */
	static function add_events()
	{
		if ( ! wp_next_scheduled( 'automatewoo_fifteen_minute_worker' ) )
		{
			wp_schedule_event( time(), 'automatewoo_fifteen_minutes', 'automatewoo_fifteen_minute_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_thirty_minute_worker' ) )
		{
			wp_schedule_event( time(), 'automatewoo_thirty_minutes', 'automatewoo_thirty_minute_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_hourly_worker' ) )
		{
			wp_schedule_event( time(), 'hourly', 'automatewoo_hourly_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_four_hourly_worker' ) )
		{
			wp_schedule_event( time(), 'automatewoo_four_hours', 'automatewoo_four_hourly_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_daily_worker' ) )
		{
			wp_schedule_event( time(), 'daily', 'automatewoo_daily_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_two_days_worker' ) )
		{
			wp_schedule_event( time(), 'automatewoo_two_days', 'automatewoo_two_days_worker' );
		}

		if ( ! wp_next_scheduled( 'automatewoo_weekly_worker' ) )
		{
			wp_schedule_event( time(), 'automatewoo_weekly', 'automatewoo_weekly_worker' );
		}
	}



	/**
	 * @param $schedules
	 *
	 * @return mixed
	 */
	static function add_schedules( $schedules )
	{
		$schedules['automatewoo_fifteen_minutes'] = array(
			'interval' => 900,
			'display' => __( 'Fifteen Minutes' )
		);

		$schedules['automatewoo_thirty_minutes'] = array(
			'interval' => 1800,
			'display' => __( 'Thirty Minutes' )
		);

		$schedules['automatewoo_two_days'] = array(
			'interval' => 172800,
			'display' => __( 'Two Days' )
		);

		$schedules['automatewoo_four_hours'] = array(
			'interval' => 14400,
			'display' => __( 'Four Hours' )
		);

		$schedules['automatewoo_weekly'] = array(
			'interval' => 604800,
			'display' => __('Once Weekly')
		);

		return $schedules;
	}


	/**
	 * Clean up an any duplicate events
	 *
	 * @todo make this a debug tool?
	 */
	static function clean_up()
	{
		$crons = _get_cron_array();
		$worker_crons = array();

		foreach ( self::$workers as $worker )
		{
			$worker_crons[ 'automatewoo_' . $worker . '_worker' ] = 0;
		}

		foreach ( $crons as $cron )
		{
			foreach ( array_keys($cron) as $event )
			{
				if ( isset( $worker_crons[$event] ) )
				{
					$worker_crons[$event]++;

					if ( $worker_crons[$event] > 1 )
					{
						// clear the event if there is more than one
						wp_clear_scheduled_hook($event);
					}
				}
			}
		}
	}

}
