<?php
/**
 * Functions for email click tracking and unsubscribes
 *
 * @class       AW_Email
 * @since       1.0.0
 * @package     AutomateWoo
 */

class AW_Email
{
	/**
	 * @var array
	 */
	public $templates = array(
		'default' => 'WooCommerce Default',
		'plain' => 'Plain Text',
	);


	/**
	 * Constructor
	 */
	public function __construct()
	{
		$this->add_content_filters();


		if ( aw_request( 'aw-unsubscribe' ) )
		{
			$this->catch_unsubscribe_url();
		}

		if ( aw_request( 'aw-click-track' ) )
		{
			$this->catch_click_track_url();
		}

		if ( aw_request( 'aw-open-track' ) )
		{
			$this->catch_open_track_url();
		}
	}


	/**
	 *
	 */
	public function add_content_filters()
	{
		add_filter( 'automatewoo_email_content', 'wptexturize' );
		add_filter( 'automatewoo_email_content', 'convert_smilies');
		add_filter( 'automatewoo_email_content', 'wpautop' );
		add_filter( 'automatewoo_email_content', 'shortcode_unautop' );
		add_filter( 'automatewoo_email_content', 'do_shortcode', 11 ); // AFTER wpautop()
	}



	/**
	 * @return array
	 */
	public function get_email_templates()
	{
		return apply_filters('automatewoo_email_templates', $this->templates );
	}



	/**
	 * Display and process form submission for unsubscribes
	 */
	public function catch_unsubscribe_url()
	{
		ob_start();

		$workflow_id = absint( aw_request( 'workflow' ) );
		$user_email = is_email( aw_request( 'user' ) );

		if ( aw_request('confirmed') )
		{
			$success = false;

			// process unsubscribe
			if ( $workflow_id && $user_email )
			{
				$workflow = new AW_Model_Workflow( $workflow_id );
				$user = get_user_by('email', $user_email );

				if ( $workflow->exists && $user instanceof WP_User )
				{
					if ( $workflow->unsubscribe_user( $user->ID ) )
					{
						$notice_type = 'success';
						$success = true;
						aw_get_template( 'unsubscribe-success.php' );
					}
				}
			}

			if ( ! $success )
			{
				$notice_type = 'error';
				aw_get_template( 'unsubscribe-error.php' );
			}

		}
		else
		{
			// Show unsubscribe form
			$notice_type = 'notice';

			aw_get_template(
				'unsubscribe-form.php',
				array(
					'unsubscribe_confirm_url' => $this->generate_unsubscribe_url( $workflow_id, $user_email, true )
				)
			);
		}

		$message = ob_get_clean();

		// Ensure notice is not added twice e.g. if redirected to ssl
		if ( ! wc_has_notice( $message, $notice_type ) )
		{
			wc_add_notice( $message, $notice_type );
		}
	}



	/**
	 *
	 */
	public function catch_click_track_url()
	{
		$redirect = urldecode(aw_request( 'redirect' ));
		$log_id = aw_request( 'log' );

		if ( ! $redirect || ! $log_id )
			return;

		$log = new AW_Model_Log( absint( $log_id ) );

		if ( $log->exists )
		{
			$log->record_click( $redirect );
		}

		wp_redirect( $redirect );
		exit;
	}


	/**
	 * @param $log_id
	 * @param $redirect
	 *
	 * @return string
	 */
	public function generate_click_track_url( $log_id, $redirect )
	{
		$url = add_query_arg(array(
			'aw-click-track' => '1',
			'log' => $log_id,
			'redirect' => urlencode( $redirect )
		), home_url() );

		return apply_filters( 'automatewoo_click_track_url', $url );
	}


	/**
	 * @param $log_id
	 *
	 * @return string
	 */
	public function generate_open_track_url( $log_id )
	{
		$url = add_query_arg(array(
			'aw-open-track' => '1',
			'log' => $log_id
		), home_url() );

		return apply_filters( 'automatewoo_open_track_url', $url );
	}


	/**
	 *
	 */
	public function catch_open_track_url()
	{
		$log_id = absint( aw_request( 'log' ) );

		if ( $log_id )
		{
			$log = new AW_Model_Log( absint( $log_id ) );

			if ( $log->exists )
			{
				$log->record_open();
			}
		}

		$image_path = AutomateWoo()->plugin_path . 'admin/assets/img/blank.gif';

		// render image
		header( 'Content-Type: image/gif' );
		header( 'Pragma: public' ); // required
		header( 'Expires: 0' ); // no cache
		header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header( 'Cache-Control: private', false );
		header( 'Content-Disposition: attachment; filename="blank.gif"' );
		header( 'Content-Transfer-Encoding: binary' );
		header( 'Content-Length: ' . filesize( $image_path ) ); // provide file size
		readfile( $image_path );
		exit;
	}


	/**
	 * Use an email here rather than an id for security
	 *
	 * @param $workflow_id
	 * @param $user_email
	 * @param bool $confirmed
	 *
	 * @return bool|string
	 */
	public function generate_unsubscribe_url( $workflow_id, $user_email, $confirmed = false )
	{
		$url = add_query_arg(array(
			'aw-unsubscribe' => '1',
			'workflow' => absint( $workflow_id ),
			'user' => urlencode( $user_email ),
			'confirmed' => $confirmed
		), wc_get_page_permalink('myaccount') );

		return apply_filters( 'automatewoo_unsubscribe_url', $url );
	}


	/**
	 * @param $input
	 * @param bool $remove_invalid
	 * @return array
	 */
	function parse_multi_email_field($input, $remove_invalid = true )
	{
		$emails = array();

		$input = preg_replace( '/\s/u', '', $input ); // remove whitespace
		$input = explode(',', $input );

		foreach ( $input as $email )
		{
			if ( ! $remove_invalid || is_email( $email ) )
			{
				$emails[] = $email;
			}
		}

		return $emails;
	}

}

