<?php
/**
 * @class       AW_Install
 * @package     AutomateWoo
 * @since       1.0.0
 */

class AW_Install
{
	/**
	 * Database updates
	 * @var array
	 */
	private static $db_updates = array(
		'2.1.0' => 'automatewoo-update-2.1.0.php',
		'2.3' => 'automatewoo-update-2.3.php',
	);


	/**
	 * Init
	 */
	public static function init()
	{
		add_action( 'admin_init', array( __CLASS__, 'admin_init' ), 5 );
		add_action( 'admin_menu', array( __CLASS__, 'admin_menu' ), 5 );
	}



	/**
	 * Admin init
	 */
	public static function admin_init()
	{
		// check version
		if ( ! defined( 'IFRAME_REQUEST' ) && ( AutomateWoo()->options->version != AutomateWoo()->version ) )
		{
			self::install();

			// check for required database update
			if ( self::database_upgrade_available() )
			{
				add_action( 'admin_notices', array( __CLASS__, 'database_upgrade_prompt') );
			}
			else
			{
				self::update_version();
			}
		}
	}



	/**
	 * Add upgrade page endpoint
	 */
	static function admin_menu()
	{
		add_submenu_page('edit.php?post_type=aw_workflow', __('AutomateWoo Data Update','automatewoo'), __('AutomateWoo Data Update','automatewoo'), 'manage_woocommerce','automatewoo_data_update',
			array( __CLASS__, 'database_upgrade_page'
		) );
	}


	/**
	 * Install
	 */
	public static function install()
	{
		self::create_tables();

		// Trigger action
		do_action( 'automatewoo_installed' );
	}


	/**
	 * @return bool
	 */
	static function database_upgrade_available()
	{
		return AutomateWoo()->options->version && version_compare( AutomateWoo()->options->version, max( array_keys( self::$db_updates ) ), '<' );
	}


	/**
	 * Handle updates
	 */
	public static function update()
	{
		@ini_set( 'memory_limit', apply_filters( 'admin_memory_limit', WP_MAX_MEMORY_LIMIT ) );

		foreach ( self::$db_updates as $version => $updater )
		{
			if ( version_compare( AutomateWoo()->options->version, $version, '<' ) )
			{
				include( AutomateWoo()->plugin_path . 'includes/updates/' . $updater );
			}
		}

		self::update_version();
	}


	/**
	 * Update version to current
	 */
	private static function update_version()
	{
		delete_option( 'automatewoo_version' );
		add_option( 'automatewoo_version', AutomateWoo()->version );
	}



	/**
	 * Set up the database tables which the plugin needs to function.
	 */
	private static function create_tables()
	{
		global $wpdb;

		$wpdb->hide_errors();

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		dbDelta( self::get_schema() );
	}


	/**
	 * Renders prompt notice for user to update
	 */
	static function database_upgrade_prompt()
	{
		AutomateWoo()->admin_helper->get_view('database-upgrade-prompt');
	}


	/**
	 * Displays upgrade page content
	 */
	static function database_upgrade_page()
	{
		AutomateWoo()->admin_helper->get_view('database-upgrade');
	}


	/**
	 * @return bool
	 */
	static function is_data_update_screen()
	{
		$screen = get_current_screen();
		return $screen->id === 'admin_page_automatewoo_data_update';
	}


	/**
	 * Get Table schema
	 * @return string
	 */
	private static function get_schema()
	{
		global $wpdb;

		$collate = '';

		if ( $wpdb->has_cap( 'collation' ) ) {
			if ( ! empty( $wpdb->charset ) ) {
				$collate .= "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$collate .= " COLLATE $wpdb->collate";
			}
		}

		return "
CREATE TABLE {$wpdb->prefix}automatewoo_guests (
  id bigint(20) NOT NULL auto_increment,
  `email` varchar(255) NOT NULL default '',
  `tracking_key` varchar(32) NOT NULL default '',
  created datetime NULL,
  last_active datetime NULL,
  `language` varchar(10) NOT NULL default '',
  PRIMARY KEY  (id)
) $collate;
CREATE TABLE {$wpdb->prefix}automatewoo_abandoned_carts (
  id bigint(20) NOT NULL auto_increment,
  user_id bigint(20) NOT NULL default 0,
  guest_id bigint(20) NOT NULL default 0,
  last_modified datetime NULL,
  items longtext NOT NULL default '',
  total varchar(32) NOT NULL default '0',
  PRIMARY KEY  (id)
) $collate;
CREATE TABLE {$wpdb->prefix}automatewoo_queue (
  id bigint(20) NOT NULL auto_increment,
  workflow_id bigint(20) NULL,
  date datetime NULL,
  data_items longtext NOT NULL default '',
  failed int(1) NOT NULL DEFAULT 0,
  PRIMARY KEY  (id)
) $collate;
CREATE TABLE {$wpdb->prefix}automatewoo_logs (
  id bigint(20) NOT NULL auto_increment,
  workflow_id bigint(20) NULL,
  date datetime NULL,
  tracking_enabled int(1) NOT NULL DEFAULT 0,
  conversion_tracking_enabled int(1) NOT NULL DEFAULT 0,
  PRIMARY KEY  (id)
) $collate;
CREATE TABLE {$wpdb->prefix}automatewoo_log_meta (
  meta_id bigint(20) NOT NULL auto_increment,
  log_id bigint(20) NULL,
  meta_key varchar(255) NULL,
  meta_value longtext NOT NULL default '',
  PRIMARY KEY  (meta_id)
) $collate;
CREATE TABLE {$wpdb->prefix}automatewoo_unsubscribes (
  id bigint(20) NOT NULL auto_increment,
  workflow_id bigint(20) NULL,
  user_id bigint(20) NOT NULL default 0,
  date datetime NULL,
  PRIMARY KEY  (id)
) $collate;
		";
	}

}
