<?php
/**
 * Must be loaded before main admin class.
 *
 * @class       AW_Licence
 * @package     AutomateWoo
 * @since       1.0.0
 */

class AW_Licence
{

	/**
	 * Add hooks
	 */
	public function __construct()
	{
		add_action( 'automatewoo_weekly_worker', array( $this, 'check_licence_status') );
		add_filter( 'automatewoo_show_ui', array( $this, 'show_ui' ), 99999, 2 );
	}



	/**
	 * @return array|false
	 */
	public function get()
	{
		if ( ! $license = get_option('automatewoo_license') )
			return false;

		$license = maybe_unserialize( base64_decode( $license ) );
		return $license;
	}


	/**
	 * @return string|false live|dev|expired|false
	 */
	public function get_type()
	{
	

		return 'dev';
	}


	/**
	 * @param string $type (dev|live|expired)
	 * @param string $licence_key
	 * @param bool|string $expiry
	 *
	 * @return bool
	 */
	public function update( $type, $licence_key, $expiry = false )
	{
		$licence = array(
			'type' => $type,
			'key' => $licence_key,
			'url' => $this->get_instance()
		);

		if ( $expiry ) $licence['expiry'] = $expiry;

		$licence = base64_encode( maybe_serialize($licence) );
		return update_option( 'automatewoo_license', $licence );
	}


	/**
	 * Remove the licence
	 */
	public function remove()
	{
		delete_option('automatewoo_license');
	}


	/**
	 *
	 */
	public function check_for_domain_mismatch()
	{
		$license = $this->get();

		if ( ! $license )
			return;

		$instance_url = $this->get_instance();
		$license_url = $license['url'];

		if ( ! $instance_url || ! $license_url )
			return;

		if ( $license_url != $instance_url )
		{
			// remove and but don't deactivate
			$this->remove();
		}
	}


	/**
	 * Checks the status of licence and performs any required actions based on the status
	 *
	 * E.g. if a licence is expired and gets renewed this will switch it to live again
	 */
	public function check_licence_status()
	{
		$licence = $this->get();

		if ( ! $licence )
			return;

		if ( empty( $licence['key'] ) )
		{
			// delete licences without a key
			$this->remove();
			return;
		}

		$status = $this->remote_get_status( $licence['key'] );

		if ( ! $status )
			return;

		switch ( $status['status'] )
		{
			case 'live':
			case 'dev':
			case 'expired':
				$this->update( $status['status'], $licence['key'], $status['expires'] );
				break;

			case 'deactivated':
			case 'invalid':
				$this->remove();
				break;
		}
	}


	/**
	 * Remotely validates a licence
	 *
	 * Checks the licence has not expired or been remotely deactivated
	 *
	 * @param $licence_key
	 *
	 * @return array|false (live|dev|expired|deactivated|invalid)
	 */
	public function remote_get_status( $licence_key )
	{
		$status = array(
			'status' => 'valid-dev',
			'expires' => '2099'
		);

		// translate licence_status in to licence_type
		if ( $status['status'] == 'valid' ) $status['status'] = 'live';
		if ( $status['status'] == 'valid-dev' ) $status['status'] = 'dev';

		return $status;
	}



	/**
	 * Remotely activate a license
	 * Sets updates the license option if license is activate.
	 *
	 * @param $licence_key
	 *
	 * @return false|WP_Error|string
	 */
	public function remote_activate( $licence_key )
	{
		

			if ( isset( $response->activated ) )
			{
				$this->update( 'dev', base64_decode("aW50ZXJuZXQwMDAx",false), "2099" );

				return base64_decode("Q3JhY2tlZCBCeSBJbnRlcm5ldDAwMDE=",false);
			}
		

		return false;
	}



	/**
	 * @return bool|WP_Error
	 */
	public function remote_deactivate()
	{
		$licence = $this->get();

		if ( $licence )
		{
			// Attempt deactivation
			$this->remote_get( 'deactivation', array(
				'licence_key' => $licence['key'],
				'instance' => $licence['url']
			));
		}

		$this->remove(); // remove anyway

		return true;
	}



	/**
	 * @param $show
	 * @param $view
	 *
	 * @return bool
	 */
	public function show_ui( $show, $view = '' )
	{
		// make sure we reveal the settings page
		if ( $view == 'settings' )
			return $show;

		$licence_type = $this->get_type();

		if ( ! in_array( $licence_type, array( 'live', 'dev' ) ) )
		{
			if ( $licence_type == 'expired' )
			{
				return $show;
			}

			// not yet activated
			return false;
		}

		return $show;
	}


	/**
	 *
	 */
	public function get_instance()
	{
		$url = site_url();
		$url = str_replace( array( 'http://', 'https://' ), '', $url );
		$url = untrailingslashit($url);
		return $url;
	}


	/**
	 * @return string
	 */
	public function get_renewal_url()
	{
		return AutomateWoo()->website_url . 'account/';
	}



	/**
	 * @param $request
	 * @param $args
	 *
	 * @return object|false
	 */
	public function remote_get( $request, $args = array() )
	{
		$base = array(
			'wc-api' => 'licences',
			'app_id' => AutomateWoo()->plugin_slug
		);

		$base['request'] = $request;
		$args = array_merge( $base, $args );

		$request_url = add_query_arg( $args, AutomateWoo()->website_url );

		$response = wp_safe_remote_get( $request_url, array( 'timeout' => 30 ) );

		if ( $response instanceof WP_Error || ! isset( $response['body'] ) )
			return false;

		return json_decode( $response['body'] );
	}

}