<?php
/**
 * @class 		AW_Mailer
 * @since		2.2
 * @package		AutomateWoo
 */

class AW_Mailer
{
	/**
	 * @var string
	 */
	public $email;

	/**
	 * @var WP_User
	 */
	public $user;

	/**
	 * @var string
	 */
	public $template;

	/**
	 * @var string
	 */
	public $heading;

	/**
	 * @var string
	 */
	public $content;

	/**
	 * @var string
	 */
	public $subject;

	/**
	 * @var AW_Model_Workflow
	 */
	public $workflow;

	/**
	 * @var string
	 */
	public $email_type = 'html';


	/**
	 * @param $subject
	 * @param $email
	 * @param $content
	 * @param string $template
	 */
	public function __construct( $subject, $email, $content, $template = 'default' )
	{
		$this->email = $email;
		$this->subject = $subject;
		$this->content = $content;
		$this->template = $template;

		$this->user = get_user_by( 'email', $email );

		// include css inliner
		if ( ! class_exists( 'Emogrifier' ) && class_exists( 'DOMDocument' ) )
		{
			include_once( WC()->plugin_path() . '/includes/libraries/class-emogrifier.php' );
		}
	}


	/**
	 * @param $heading
	 */
	public function set_heading( $heading )
	{
		$this->heading = $heading;
	}


	/**
	 * @param $workflow
	 */
	public function set_workflow( $workflow )
	{
		$this->workflow = $workflow;
	}



	/**
	 * Generate HTML, inline CSS and send email
	 *
	 * @return true|WP_Error
	 */
	public function send()
	{
		if ( ! $this->email )
		{
			return new WP_Error( 2, __( 'The to email field was blank.', 'automatewoo' ) );
		}

		if ( ! is_email( $this->email ) )
		{
			return new WP_Error( 1, sprintf(__( "'%s' is not a valid email.", 'automatewoo' ), $this->email ) );
		}

		if ( $this->user_is_unsubscribed() )
		{
			return new WP_Error( 2, __( 'The user is unsubscribed from this workflow.', 'automatewoo' ) );
		}

		add_filter( 'wp_mail_from', array( $this, 'get_from_address' ) );
		add_filter( 'wp_mail_from_name', array( $this, 'get_from_name' ) );
		add_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );

		$content = apply_filters( 'woocommerce_mail_content', $this->style_inline( $this->get_html() ) );
		$sent = wp_mail( $this->email, $this->subject, $content, "Content-Type: " . $this->get_content_type() . "\r\n" );

		remove_filter( 'wp_mail_from', array( $this, 'get_from_address' ) );
		remove_filter( 'wp_mail_from_name', array( $this, 'get_from_name' ) );
		remove_filter( 'wp_mail_content_type', array( $this, 'get_content_type' ) );

		if ( $sent === false )
		{
			return new WP_Error( 2, __( 'The wp_mail() function returned false.', 'automatewoo' ) );
		}

		return $sent;
	}



	/**
	 * Returns html without CSS inline
	 *
	 * @return string
	 */
	public function get_html()
	{
		if ( $this->user ) add_filter( 'woocommerce_email_footer_text', array( $this, '_email_footer_text' ) );

		AW_Mailer_API::$mailer = $this;
		$this->prepare_content();

		// Buffer
		ob_start();

		$this->get_template_part('email-header.php', array( 'email_heading' => $this->heading ) );

		echo $this->content;

		$this->get_template_part('email-footer.php' );

		$html = ob_get_clean();

		if ( $this->user ) remove_filter( 'woocommerce_email_footer_text', array( $this, '_email_footer_text' ) );

		AW_Mailer_API::$mailer = false;

		return $html;
	}


	/**
	 *
	 */
	public function prepare_content()
	{
		if ( $this->workflow && $this->workflow->is_tracking_enabled() )
		{
			$this->content = $this->replace_urls_with_tracking_urls( $this->content );
			$this->content = $this->append_tracking_pixel_to_content( $this->content );
		}

		// pass through content filters to convert short codes etc
		// IMPORTANT do this after URLs are modified so entities are not encoded
		$this->content = apply_filters( 'automatewoo_email_content', $this->content );
	}



	/**
	 * Apply inline styles to dynamic content.
	 *
	 * @param string|null $content
	 * @return string
	 */
	public function style_inline( $content )
	{
		if ( ! class_exists( 'DOMDocument' ) ) return $content;

		ob_start();
		aw_get_template( 'email/styles.php' );
		$this->get_template_part( 'email-styles.php' );
		$css = ob_get_clean();

		try
		{
			$emogrifier = new Emogrifier( $content, $css );
			$content    = $emogrifier->emogrify();
		}
		catch ( Exception $e )
		{
			$logger = new WC_Logger();
			$logger->add( 'emogrifier', $e->getMessage() );
		}

		return $content;
	}


	/**
	 * @return bool
	 */
	public function user_is_unsubscribed()
	{
		if ( ! $this->user || ! $this->workflow ) return false;
		return $this->workflow->is_user_unsubscribed( $this->user->ID );
	}


	/**
	 * @param $text
	 * @return string
	 */
	function _email_footer_text( $text )
	{
		if ( empty( $this->user ) )
			return $text;

		$unsubscribe_url = $this->get_unsubscribe_url();
		$unsubscribe_text = apply_filters( 'automatewoo_email_unsubscribe_text', __( 'Unsubscribe', 'automatewoo' ) );

		// add separator if there is footer text
		if ( trim( $text ) )
		{
			$text .= apply_filters( 'automatewoo_email_footer_separator',  ' - ' );
		}

		$text .= sprintf(__('<a href="%s">'.$unsubscribe_text.'</a>'), $unsubscribe_url );

		return $text;
	}



	/**
	 * Use an email here rather than an id for security
	 *
	 * @return bool|string
	 */
	public function get_unsubscribe_url()
	{
		return AutomateWoo()->email->generate_unsubscribe_url( $this->workflow->id, $this->email, false );
	}



	/**
	 * @param $file
	 * @param array $args
	 *
	 * @return bool
	 */
	public function get_template_part( $file, $args = array() )
	{
		if ( $args && is_array( $args ) ) {
			extract( $args );
		}

		switch( $this->template )
		{
			case 'default':
				$template_name = 'emails/' . $file;
				$template_path = '';
				break;

			case 'plain':
				return false;
				break;

			default:
				$template_name = $file;
				$template_path = 'automatewoo/custom-email-templates/'. $this->template;
				break;
		}

		$located = wc_locate_template( $template_name, $template_path );

		if ( file_exists( $located ) ) include( $located );
	}



	/**
	 * @param $content string
	 *
	 * @return string
	 */
	public function replace_urls_with_tracking_urls( $content )
	{
		$replacer = new AW_Replace_Helper( $content, array( $this, '_replace_urls_preg_callback' ), 'href_urls' );
		return $replacer->process();
	}


	/**
	 * @param $content
	 *
	 * @return string
	 */
	public function append_tracking_pixel_to_content( $content )
	{
		$tracking_pixel = '<img src="' . AutomateWoo()->email->generate_open_track_url( $this->workflow->log->id ) . '" height="1" width="1" alt="" style="display:inline">';

		return $content . $tracking_pixel;
	}


	/**
	 * @param $url
	 *
	 * @return string
	 */
	function _replace_urls_preg_callback( $url )
	{
		if ( ! $url || ! $this->workflow->log->id ) return $url;

		$url = $this->workflow->append_ga_tracking_to_url( $url );

		return 'href="' . AutomateWoo()->email->generate_click_track_url( $this->workflow->log->id, $url ) . '"';
	}



	/**
	 * Get the from name for outgoing emails.
	 * @return string
	 */
	public function get_from_name()
	{
		$from_name = apply_filters( 'woocommerce_email_from_name', get_option( 'woocommerce_email_from_name' ), $this );
		return wp_specialchars_decode( esc_html( $from_name ), ENT_QUOTES );
	}


	/**
	 * Get the from address for outgoing emails.
	 * @return string
	 */
	public function get_from_address()
	{
		$from_address = apply_filters( 'woocommerce_email_from_address', get_option( 'woocommerce_email_from_address' ), $this );
		return sanitize_email( $from_address );
	}


	/**
	 * get_type function.
	 *
	 * @return string
	 */
	public function get_email_type()
	{
		return $this->email_type && class_exists( 'DOMDocument' ) ? $this->email_type : 'plain';
	}


	/**
	 * Get email content type.
	 *
	 * @return string
	 */
	public function get_content_type()
	{
		switch ( $this->get_email_type() )
		{
			case 'html' :
				return 'text/html';
			case 'multipart' :
				return 'multipart/alternative';
			default :
				return 'text/plain';
		}
	}


}