<?php
/**
 * Used to create new queued events and use existing
 *
 * @class       AW_Model_Queued_Event
 * @package     AutomateWoo/Models
 * @since       2.1.0
 *
 * @property int $workflow_id
 * @property string $date UTC
 * @property array $data_items
 * @property bool $failed
 */

class AW_Model_Queued_Event extends AW_Model
{
	/**
	 * @var bool|array
	 */
	private $uncompressed_data_items;

	/**
	 * @var AW_Model_Workflow
	 */
	private $workflow;

	/**
	 * @var bool
	 */
	private $workflow_loaded = false;


	/**
	 * @param bool|int $id
	 */
	public function __construct( $id = false )
	{
		$this->table_name = AutomateWoo()->table_name_queue;

		if ( $id ) $this->get_by( 'id', $id );
	}



	/**
	 * TODO make hookable
	 *
	 * @param $data_items
	 */
	public function set_data_items( $data_items )
	{
		$this->uncompressed_data_items = $data_items;

		$data_items = array();

		foreach ( $this->uncompressed_data_items as $data_type => $data_item )
		{
			if ( aw_validate_data_item( $data_type, $data_item ) )
			{
				switch( $data_type )
				{
					case 'user':
						$data_items[$data_type] = $data_item->ID;
						break;

					case 'order':
					case 'product':
					case 'subscription':
					case 'order_note':
					case 'wishlist':
						$data_items[$data_type] = $data_item->id;
						break;

					case 'category':
						$data_items[$data_type] = $data_item->term_id;
						break;

					case 'comment':
						$data_items[$data_type] = $data_item->comment_ID;
						break;

					case 'guest':
						$data_items[$data_type] = 0;
						break;

					case 'cart':
						// Carts don't really need to be stored
						break;
				}
			}
		}

		// Save
		$this->data_items = $data_items;
	}


	/**
	 * @todo add order_item
	 *
	 * @return array
	 */
	public function get_data_items()
	{
		// already uncompressed
		if ( is_array( $this->uncompressed_data_items ) )
			return $this->uncompressed_data_items;

		$this->uncompressed_data_items = array();

		if ( ! $this->data_items ) return array();

		$compressed_data_items = $this->data_items;

		foreach ( $compressed_data_items as $data_type => $data_item_id )
		{
			switch( $data_type )
			{
				case 'user':

					$user_id = $data_item_id;

					// Support for guest orders, create guest user from order
					if ( $user_id == 0 && isset( $compressed_data_items['order'] ) )
					{
						$order = wc_get_order( $compressed_data_items['order'] );
						$this->uncompressed_data_items[$data_type] = new AW_Model_Order_Guest($order);
					}
					elseif ( $user_id )
					{
						$this->uncompressed_data_items[$data_type] = get_user_by( 'id', $data_item_id );
					}

					break;

				case 'order':
					$this->uncompressed_data_items[$data_type] = wc_get_order( $data_item_id );
					break;

				case 'wishlist':
					$this->uncompressed_data_items[$data_type] = AutomateWoo()->wishlist->get_wishlist($data_item_id);
					break;

				case 'product':
					$this->uncompressed_data_items[$data_type] = wc_get_product($data_item_id);
					break;

				case 'category':
					$this->uncompressed_data_items[$data_type] = get_term( $data_item_id, 'product_cat' );
					break;

				case 'subscription':
					if ( AutomateWoo()->integrations->subscriptions_enabled() )
					{
						$this->uncompressed_data_items[$data_type] = wcs_get_subscription($data_item_id);
					}
					break;

				case 'comment':
					$this->uncompressed_data_items[$data_type] = get_comment($data_item_id);
					break;

				case 'order_note':
					if ( $comment = get_comment($data_item_id) )
					{
						$this->uncompressed_data_items[$data_type] = new AW_Model_Order_Note( $comment->comment_ID, $comment->comment_content, $comment->comment_post_ID );
					}
					break;

				case 'guest':

					// Guests are stored by their email not their ID

					if ( is_email( $data_item_id ) )
					{
						$guest_email = $data_item_id;
					}
					elseif ( isset( $compressed_data_items['comment'] ) )
					{
						// If there is a comment fetch the guest info from that
						$comment = get_comment( $compressed_data_items['comment'] );
						$guest_email = $comment->comment_author_email;
					}
					else
					{
						continue;
					}

					$guest = new AW_Model_Guest();
					$guest->get_by( 'email', $guest_email );

					if ( ! $guest->exists )
					{
						$guest->email = $guest_email;
					}

					// Yes we will still pass the guest object even if it doesn't exist in the database.
					// In most cases it should have been stored but there is no harm since all we need is an email

					$this->uncompressed_data_items[$data_type] = $guest;

					break;

				case 'cart':
					// Carts don't really need to be stored
					break;
			}
		}

		return $this->uncompressed_data_items;
	}



	/**
	 * Data items should be retrieved through the workflow
	 *
	 * @return AW_Model_Workflow
	 */
	public function get_workflow()
	{
		if ( ! $this->workflow_loaded )
		{
			$this->workflow = new AW_Model_Workflow( get_post( $this->workflow_id ) );
			$this->workflow->set_data_items( $this->get_data_items() );
			$this->workflow_loaded = true;
		}

		return $this->workflow;
	}



	/**
	 * @param $value int
	 * @param $unit string (h,d,w)
	 *
	 * @return DateTime
	 */
	public function calculate_delay( $value, $unit )
	{
		$date = new DateTime();
		$date->setTimestamp( current_time('timestamp', true ) ); // UTC

		switch( $unit )
		{
			case 'm':
				$date->modify("+$value minutes");
				break;

			case 'h':
				$date->modify("+$value hours");
				break;

			case 'd':
				$date->modify("+$value days");
				break;

			case 'w':
				$date->modify("+$value weeks");
				break;
		}

		return $date;
	}


	/**
	 * @param $date DateTime
	 */
	public function set_date( $date )
	{
		if ( ! $date instanceof DateTime )
			return;

		$this->date = $date->format('Y-m-d H:i:s');
	}



	/**
	 * @return bool
	 */
	public function get_data_items_still_exist()
	{
		$data_items = $this->get_data_items();

		foreach ( $data_items as $data_item )
		{
			if ( ! $data_item )
				return false;
		}

		return true;
	}


	/**
	 *
	 */
	public function run()
	{
		if ( ! $this->exists ) return false;

		$workflow = $this->get_workflow();

		if ( $workflow->is_active() && $this->get_data_items_still_exist() )
		{
			$workflow->run( true );
			$this->delete();
			return true;
		}
		else
		{
			$this->failed = true;
			$this->save();
			return false;
		}
	}


	/**
	 *
	 */
	public function clear_cached_data()
	{
		if ( ! $this->workflow_id )
			return;

		AutomateWoo()->cache->delete( 'current_queue_count/workflow=' . $this->workflow_id );
	}


	/**
	 *
	 */
	public function save()
	{
		$this->clear_cached_data();
		parent::save();
	}


	/**
	 *
	 */
	public function delete()
	{
		$this->clear_cached_data();
		parent::delete();
	}


}

