<?php
/**
 * The run() should be only called by a trigger which then calls actions.
 *
 * NOTE: a rule only has 1 trigger but can have multiple actions.
 * Maybe add support for multiple triggers later.
 *
 * @package     AutomateWoo/Models
 * @class       AW_Model_Workflow
 * @since       1.0.0
 */
class AW_Model_Workflow
{
	/**
	 * @var int
	 */
	public $id;

	/**
	 * @var object post
	 */
	public $post;

	/**
	 * @var string
	 */
	public $title;

	/**
	 * @var int
	 */
	public $times_run;

	/**
	 * @var AW_Trigger
	 */
	public $trigger;

	/**
	 * @var array
	 */
	public $actions;

	/**
	 * Data items passed by the trigger through to the action
	 * eg a user objects
	 * @var array
	 */
	public $data_items;

	/**
	 * @var bool
	 */
	public $trigger_loaded;

	/**
	 * @var bool
	 */
	public $actions_loaded;

	/**
	 * @var array
	 */
	public $options = array();

	/**
	 * @var bool
	 */
	public $options_loaded;

	/**
	 * @var AW_Variable_Processor
	 */
	private $variable_processor;

	/**
	 * @var AW_Model_Log
	 */
	public $log;

	/**
	 * @var bool
	 */
	public $exists = false;

	/**
	 * @var bool
	 */
	public $preview_mode = false;


	/**
	 * @param $post mixed (object or post ID)
	 */
	public function __construct( $post )
	{
		if ( ! $post instanceof WP_Post )
		{
			// Get from id
			$post = get_post($post);
		}

		// workflow doesn't exists
		if ( ! $post )
			return;

		$this->exists = true;
		$this->post = $post;
		$this->id = $post->ID;
		$this->title = $post->post_title;
	}



	/**
	 * Get trigger
	 */
	public function load_trigger()
	{
		if ( $this->trigger_loaded )
			return;

		$this->trigger_loaded = true;

		$trigger_name = get_post_meta( $this->id, 'trigger_name', true );

		if ( ! $trigger_name )
		{
			$this->trigger = false;
			return;
		}

		if ( ! AutomateWoo()->get_registered_trigger($trigger_name) )
			return;

		$this->trigger = clone AutomateWoo()->get_registered_trigger($trigger_name);
		$this->trigger->set_options( get_post_meta( $this->id, 'trigger_options', true ) );
	}


	/**
	 * Get actions names from meta
	 */
	public function load_actions()
	{
		if ( $this->actions_loaded )
			return;

		$this->actions_loaded = true;

		$actions_data = get_post_meta( $this->id, 'actions', true );

		if ( ! $actions_data || ! is_array( $actions_data ) )
		{
			$this->actions = false;
			return;
		}


		$n = 1;
		foreach ( $actions_data as $action )
		{
			if ( ! isset( $action['action_name'] ) )
				continue;

			if ( ! AutomateWoo()->get_registered_action( $action['action_name'] ) )
				continue;

			// Create an AW_Action object from the stored data
			$action_obj = clone AutomateWoo()->get_registered_action( $action['action_name'] );
			$action_obj->set_options( $action );
			$this->actions[$n] = $action_obj;
			$n++;
		}
	}


	/**
	 * @return AW_Variable_Processor
	 */
	public function variable_processor()
	{
		if ( ! $this->variable_processor instanceof AW_Variable_Processor )
			$this->variable_processor = new AW_Variable_Processor( $this );

		return $this->variable_processor;
	}


	/**
	 * Returns the trigger that caused this rule to run
	 *
	 * @return AW_Trigger
	 */
	public function get_trigger()
	{
		if ( ! $this->trigger_loaded )
			$this->load_trigger();

		return $this->trigger;
	}



	/**
	 * Returns the saved actions with their data
	 *
	 * @return object
	 */
	public function get_actions()
	{
		if ( ! $this->actions_loaded )
			$this->load_actions();

		return $this->actions;
	}


	/**
	 * Returns the saved actions with their data
	 *
	 * @param $number
	 *
	 * @return object
	 */
	public function get_action( $number )
	{
		if ( ! $this->actions_loaded )
			$this->load_actions();

		if ( ! isset( $this->actions[$number] ) )
			return false;

		return $this->actions[$number];
	}


	/**
	 * Will run or queue the workflow based on its options
	 *
	 * @param bool $from_queue
	 *
	 * @return bool
	 */
	public function run( $from_queue = false )
	{
		$when_to_run = $this->get_option('when_to_run');

		// Run now?
		if ( $when_to_run == 'immediately' || $from_queue )
		{
			if ( $from_queue )
			{
				$trigger = $this->get_trigger();
				$still_valid = $trigger->validate_before_queued_event( $this );

				if ( ! $still_valid )
					return false;
			}


			// Run now

			// Create log
			$this->create_run_log();

			if ( $this->get_actions() )
			{
				foreach ( $this->get_actions() as $action )
				{
					$action->workflow = $this;

					do_action('automatewoo_before_action_run', $action, $this );

					$action->run();

					do_action('automatewoo_after_action_run', $action, $this );

				}
			}

			do_action( 'automatewoo_after_workflow_run', $this );

		}
		else
		{
			// Add to queue

			$queue = new AW_Model_Queued_Event();
			$queue->set_data_items( $this->data_items );
			$queue->workflow_id = $this->id;

			if ( $when_to_run == 'delayed' )
			{
				$date = $queue->calculate_delay( absint( $this->get_option('run_delay_value') ), $this->get_option('run_delay_unit') );
			}
			elseif ( $when_to_run == 'datetime' )
			{
				$datetime = $this->get_option('queue_datetime', true );
				if ( $datetime )
				{
					$timestamp = strtotime( $this->get_option('queue_datetime', true ), current_time('timestamp' ) );

					$date = new DateTime();
					$date->setTimestamp($timestamp);

					// convert to UTC
					$utc_date_string = get_gmt_from_date( $date->format('Y-m-d H:i:s') );
					$date = new DateTime( $utc_date_string );
				}
				else
				{
					// if datetime is empty do not queue the item
				}
			}


			if ( ! empty( $date ) )
			{
				$queue->set_date( $date );
				$queue->save();
			}

		}

		return true;
	}



	/**
	 * Record that the workflow has been run
	 */
	public function create_run_log()
	{
		// Increment times run meta field
		$times_run = $this->get_times_run();
		$this->set_times_run( $times_run + 1 );


		// Create log
		$this->log = new AW_Model_Log();
		$this->log->workflow_id = $this->id;
		$this->log->date = current_time( 'mysql', true );

		if ( $this->get_option('click_tracking') )
		{
			$this->log->tracking_enabled = true;

			if ( $this->get_option('conversion_tracking') )
			{
				$this->log->conversion_tracking_enabled = true;
			}
		}

		$this->log->save(); // must be saved before adding meta


		if ( $order = $this->get_data_item('order') )
			$this->log->add_meta( 'order_id', $order->id );


		// special rules for orders placed by guest
		if ( $user = $this->get_data_item('user') )
		{
			if ( $user->ID === 0 ) // order placed by guest
			{
				$this->log->add_meta( 'guest_email', $user->user_email );
			}
			else
			{
				$this->log->add_meta( 'user_id', $user->ID );
			}
		}


		if ( $product = $this->get_data_item('product') )
			$this->log->add_meta( 'product_id', $product->id );

		if ( $category = $this->get_data_item('category') )
			$this->log->add_meta( 'category_id', $category->term_id );

		if ( $tag = $this->get_data_item('tag') )
			$this->log->add_meta( 'tag_id', $tag->term_id );

		if ( $wishlist = $this->get_data_item('wishlist') )
			$this->log->add_meta( 'wishlist_id', $wishlist->id );

		if ( $cart = $this->get_data_item('cart') )
			$this->log->add_meta( 'cart_id', $cart->id );

		if ( $guest = $this->get_data_item('guest') )
			$this->log->update_meta( 'guest_email', $guest->email ); // leave as update_meta just in case

		if ( $order_item = $this->get_data_item('order_item') )
			$this->log->add_meta( 'order_item_id', $order_item['id'] );

		if ( $order_note = $this->get_data_item('order_note') )
			$this->log->add_meta( 'order_note_id', $order_note->id );

		if ( $subscription = $this->get_data_item('subscription') )
			$this->log->add_meta( 'subscription_id', $subscription->id );

		if ( $comment = $this->get_data_item('comment') )
			$this->log->add_meta( 'comment_id', $comment->comment_ID );

		do_action( 'automatewoo_create_run_log', $this->log, $this );
	}



	/**
	 * @return int
	 */
	public function get_times_run()
	{
		if ( $this->times_run )
			return $this->times_run;

		return $this->times_run = absint( get_post_meta( $this->id, 'times_run', true ) );
	}


	/**
	 * @param bool $try_cache
	 * @return int
	 */
	public function get_current_queue_count( $try_cache = true )
	{
		$cache_key = 'current_queue_count/workflow=' . $this->id;
		$cache = AutomateWoo()->cache->get( $cache_key );

		if ( $try_cache && $cache !== false )
		{
			return $cache;
		}
		else
		{
			$query = new AW_Query_Queue();
			$query->where( 'workflow_id', $this->id );
			$results = $query->get_results();
			$count = $results ? count( $results ) : '-';

			AutomateWoo()->cache->set( $cache_key, $count, 168 );

			return $count;
		}
	}


	/**
	 * @param $times_run
	 */
	public function set_times_run( $times_run )
	{
		update_post_meta( $this->id, 'times_run', absint( $times_run ) );
	}


	/**
	 * @param string $name
	 * @param bool $replace_vars
	 *
	 * @return bool
	 */
	public function get_option( $name, $replace_vars = false )
	{
		if ( ! $this->options_loaded )
		{
			$this->options = get_post_meta( $this->id, 'workflow_options', true );
			$this->options_loaded = true;
		}

		if ( ! isset( $this->options[$name] ) )
			return false;

		if ( $replace_vars )
		{
			return $this->variable_processor()->process_field( $this->options[$name] );
		}

		return $this->options[$name];
	}


	/**
	 * @param $user_id
	 *
	 * Store time unsubscribed for tracking purposes
	 *
	 * @return bool
	 */
	public function unsubscribe_user( $user_id )
	{
		if ( ! $user_id )
			return false;

		$unsubscribe = new AW_Model_Unsubscribe();
		$unsubscribe->user_id = $user_id;
		$unsubscribe->workflow_id = $this->id;
		$unsubscribe->date = current_time( 'mysql', true );
		$unsubscribe->save();

		return true;
	}


	/**
	 * @param $user_id
	 *
	 * @return bool
	 */
	public function is_user_unsubscribed( $user_id )
	{
		$unsubscribe_query = new AW_Query_Unsubscribes();
		$unsubscribe_query->set_limit(1);
		$unsubscribe_query->where('workflow_id', $this->id);
		$unsubscribe_query->where('user_id', $user_id);
		return $unsubscribe_query->get_results();
	}



	/**
	 * @param $user WP_User or guest user
	 *
	 * @return bool
	 */
	public function is_first_run_for_user( $user )
	{
		return $this->get_times_run_for_user( $user ) === 0;
	}


	/**
	 * Counts items in log and in queue for this user and workflow
	 *
	 * @param $user WP_User or AW_Model_Guest_Order
	 *
	 * @return int
	 */
	public function get_times_run_for_user( $user )
	{
		$log_query = new AW_Query_Logs();
		$log_query->where( 'workflow_id', $this->id );

		if ( $user->ID === 0 ) // guest user
		{
			$log_query->where( 'guest_email', $user->user_email );
		}
		else
		{
			$log_query->where( 'user_id', $user->ID );
		}

		if ( $results = $log_query->get_results() )
			return count($results);

		return 0;
	}



	/**
	 * Counts items in log and in queue for this guest and workflow
	 *
	 * @param $guest AW_Model_Guest
	 *
	 * @return int
	 */
	public function get_times_run_for_guest( $guest )
	{
		$log_query = new AW_Query_Logs();
		$log_query->where( 'workflow_id', $this->id );
		$log_query->where( 'guest_email', $guest->email );

		if ( $results = $log_query->get_results() )
			return count($results);

		return 0;
	}



	/**
	 * @param $name
	 * @param $item
	 */
	function add_data_item( $name, $item )
	{
		$this->data_items[$name] = $item;
	}



	/**
	 * @param $data_items
	 */
	function set_data_items( $data_items )
	{
		if ( is_array( $data_items ) )
			$this->data_items = $data_items;
	}


	/**
	 * Retrieve and validate a data item
	 *
	 * @param $name string
	 *
	 * @return mixed
	 */
	function get_data_item( $name )
	{
		if ( ! isset( $this->data_items[$name] ) )
			return false;

		$item = $this->data_items[$name];

		return aw_validate_data_item( $name, $item );
	}


	/**
	 * @return bool
	 */
	function is_active()
	{
		if ( ! $this->exists ) return false;

		return $this->post->post_status == 'publish';
	}


	/**
	 * @return bool
	 */
	function is_tracking_enabled()
	{
		return $this->get_option('click_tracking');
	}


	/**
	 * @return bool
	 */
	function is_ga_tracking_enabled()
	{
		return ( $this->is_tracking_enabled() && $this->get_option('ga_link_tracking') ) ;
	}


	/**
	 * @param string $url
	 * @return string
	 */
	function append_ga_tracking_to_url( $url )
	{
		if ( empty( $url ) || ! $this->is_ga_tracking_enabled() )
			return $url;

		$url .=  strstr( $url, '?' ) ? '&' : '?';
		$url .= $this->get_option('ga_link_tracking');

		return $url;
	}


	/**
	 * @return false|string
	 */
	function get_language()
	{
		if ( AutomateWoo()->integrations->is_wpml() )
		{
			$info = wpml_get_language_information( null, $this->id );
			if ( is_array( $info ) )
				return $info['language_code'];
		}
	}


	/**
	 * Return array with all versions of this workflow including the original
	 * @return array
	 */
	function get_translation_ids()
	{
		if ( ! AutomateWoo()->integrations->is_wpml() )
		{
			return array( $this->id );
		}

		global $sitepress;

		$ids = array();

		$translations = $sitepress->get_element_translations( $this->id, 'post_post', false, true );

		if ( is_array($translations) ) foreach ( $translations as $translation )
		{
			$ids[] = $translation->element_id;
		}

		return $ids;
	}



	/**
	 *
	 */
	function enable_preview_mode()
	{
		$this->preview_mode = true;
		$this->fill_with_preview_data();
	}


	/**
	 * Fill workflow with preview data
	 */
	function fill_with_preview_data()
	{
		$this->log = new AW_Model_Log();
		$this->log->id = 1;


		/**
		 * User
		 */
		$this->add_data_item( 'user', wp_get_current_user() );


		/**
		 * Order
		 */
		$orders = new WP_Query(array(
			'post_type' => 'shop_order',
			'posts_per_page' => 1,
			'post_status' => 'wc-complete',
			'fields' => 'ids'
		));

		$this->add_data_item( 'order', wc_get_order( $orders->posts[0] ) );


		/**
		 * Product
		 */
		$product_query = new WP_Query(array(
			'post_type' => 'product',
			'posts_per_page' => 4,
			//'orderby' => 'rand',
			'fields' => 'ids'
		));
		$this->add_data_item( 'product', wc_get_product( $product_query->posts[0] ) );


		/**
		 * Category
		 */
		$cats = get_terms('product_cat', array(
			'order' => 'count',
			'number' => 1
		));

		$this->add_data_item( 'category', current($cats) );


		/**
		 * Cart
		 */
		$cart = new AW_Model_Abandoned_Cart();
		$cart->id = 1;
		$cart->total = 100;

		$cart->last_modified = current_time('mysql', true );

		$items = array();

		foreach ( $product_query->posts as $product_id )
		{
			$product = wc_get_product( $product_id );

			if ( $product->is_type('variable') )
			{
				$variations = $product->get_available_variations();
				$variation_id = $variations[0]['variation_id'];
				$variation = $variations[0]['attributes'];
			}
			else
			{
				$variation_id = 0;
				$variation = array();

			}

			$items[] = array(
				'product_id' => $product_id,
				'variation_id' => $variation_id,
				'variation' => $variation,
				'quantity' => 1,
				'line_total' => $product->get_price(),
				'line_subtotal' => $product->get_price(),
				'line_tax' => 0,
				'line_subtotal_tax' => 0,
			);
		}

		$cart->items = $items;

		$this->add_data_item( 'cart', $cart );


		/**
		 * Wishlist
		 */
		$wishlist = new stdClass();
		$wishlist->items = $product_query->posts;

		$integration = AutomateWoo()->wishlist->get_integration();

		if ( $integration == 'yith' )
		{
			$wishlist->permalink = YITH_WCWL()->get_wishlist_url();
		}
		else
		{
			$wishlist->permalink = home_url() . '/% wishlist-url-goes-here %/';
		}

		$this->add_data_item( 'wishlist', $wishlist );


		$guest = new AW_Model_Guest();
		$guest->email = 'guest@example.com';
		$this->add_data_item('guest', $guest );


		if ( AutomateWoo()->integrations->subscriptions_enabled() )
		{
			/**
			 * Subscription
			 */
			$subscriptions = wcs_get_subscriptions(array(
				'subscriptions_per_page' => 1
			));

			$this->add_data_item('subscription', current($subscriptions) );
		}
	}

}
