<?php
/**
 * Track guests across sessions via cookies. Store their info in the database.
 * Loaded on wc_init
 *
 * @class 		AW_Session_Tracker
 */

class AW_Session_Tracker
{
	/** @var int (days) */
	private $tracking_cookie_expiry;

	/** cookie name */
	private $tracking_key_cookie_name;

	/** @var string */
	private $tracking_key;

	/** @var int */
	private $user_id = 0;

	/** @var bool */
	private $_user_id_loaded = false;

	/** @var AW_Model_Guest */
	private $guest;

	/** @var bool */
	private $_guest_loaded = false;

	/**
	 * Ensures the visitor id cookie is set and matched with id stored in the database for logged in users
	 */
	public function __construct()
	{
		$this->tracking_key_cookie_name = 'wp_automatewoo_visitor_' . COOKIEHASH;
		$this->tracking_cookie_expiry = apply_filters('automatewoo_visitor_tracking_cookie_expiry', 730 ); // 2 years

		add_action( 'wp_loaded', array( $this, 'init' ) ); // Get session info after WP and plugins are loaded.
		add_action( 'wp', array( $this, 'maybe_set_session_cookies' ), 99 ); // Set cookies

		add_action( 'comment_post', array( $this, 'capture_from_comment' ), 10, 2 );
		add_action( 'automatewoo_capture_guest_email', array( $this, 'store_guest' ) ); // for third-party
	}


	/**
	 *
	 */
	public function init()
	{
		if ( isset( $_COOKIE[$this->tracking_key_cookie_name] ) )
		{
			$this->tracking_key = wc_clean( $_COOKIE[$this->tracking_key_cookie_name] );
		}
		else
		{
			$this->tracking_key = $this->generate_key();
		}


		if ( is_user_logged_in() )
		{
			// check for stored key in database
			$stored_tracking_key = get_user_meta( get_current_user_id(), 'automatewoo_visitor_key', true );

			if ( $stored_tracking_key )
			{
				// Ensure the visitor ID in cookie matches the ID in the database
				$this->tracking_key = $stored_tracking_key;
			}
			else
			{
				// store the key
				update_user_meta( get_current_user_id(), 'automatewoo_visitor_key', $this->tracking_key );
			}
		}


		// is this a new session?
		if ( empty( $_COOKIE[ 'wp_automatewoo_session_started' ] ) )
		{
			do_action( 'automatewoo_new_session_initiated' );
		}

		do_action( 'automatewoo_session_initiated' );
	}


	/**
	 *
	 */
	public function maybe_set_session_cookies()
	{
		if ( ! headers_sent() && did_action( 'wp_loaded' ) )
		{
			wc_setcookie( $this->tracking_key_cookie_name, $this->tracking_key, time() + 60 * 60 * 24 * $this->tracking_cookie_expiry );
			wc_setcookie( 'wp_automatewoo_session_started', 1 );
		}
	}


	/**
	 * Returns false if not id is set
	 *
	 * @return string|false
	 */
	public function get_tracking_key()
	{
		if ( ! did_action( 'automatewoo_session_initiated' ) )
		{
			_doing_it_wrong( __FUNCTION__, __( "Can't get tracking key before session is initiated.", 'automatewoo'), '2.3' );
			return false;
		}

		return $this->tracking_key;
	}



	/**
	 * @return string
	 */
	public function generate_key()
	{
		return wp_generate_password( 32, false );
	}


	/**
	 * Detect the user id based from their visitor id
	 *
	 * Returns false if no use is detected.
	 *
	 * @return int|false
	 */
	public function get_detected_user_id()
	{
		if ( is_user_logged_in() )
			return get_current_user_id();

		if ( ! $this->_user_id_loaded && $tracking_key = $this->get_tracking_key() )
		{
			// query the database for a matching tracking key
			$user_query = new WP_User_Query(array(
				'meta_query' => array(
					array(
						'key' => 'automatewoo_visitor_key',
						'value' => $tracking_key
					)
				),
				'fields' => 'ids'
			));

			$results = $user_query->get_results();

			if ( ! empty( $results ) )
			{
				$this->user_id = $results[0];
			}

			$this->_user_id_loaded = true;
		}

		return $this->user_id;
	}


	/**
	 * @return AW_Model_Guest|bool
	 */
	public function get_current_guest()
	{
		if ( ! $this->_guest_loaded && $tracking_key = $this->get_tracking_key() )
		{
			$guest = new AW_Model_Guest();
			$guest->get_by( 'tracking_key', $tracking_key );

			if ( $guest->exists )
			{
				$this->guest = $guest;
			}

			$this->_guest_loaded = true;
		}

		return $this->guest;
	}



	/**
	 * @param $email
	 * @return AW_Model_Guest|false
	 */
	public function store_guest( $email )
	{
		if ( ! is_email( $email ) )
			return false;

		// Check if the guest already stored
		$guest = $this->get_current_guest();

		if ( $guest )
		{
			// update email if changed
			if ( $guest->email !== $email )
				$guest->email = $email;

			$guest->save();

			return $guest;
		}
		else
		{
			$guest = new AW_Model_Guest();
			$guest->set_email( $email );
			$guest->set_tracking_key( $this->get_tracking_key() );
			$guest->set_created( current_time( 'mysql', true ) );
			$guest->set_last_active( current_time( 'mysql', true ) );
			$guest->save();

			do_action( 'automatewoo/session_tracker/new_stored_guest', $guest );

			return $guest;
		}
	}


	/**
	 * Store guest info if they place a comment
	 * @param $comment_ID
	 */
	public function capture_from_comment( $comment_ID )
	{
		$comment = get_comment( $comment_ID );

		if ( $comment && ! $comment->user_id )
		{
			$this->store_guest( $comment->comment_author_email );
		}

	}

}