<?php
/**
 * @class       AW_Trigger_Guest_Abandoned_Cart
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_Abandoned_Cart_Guest extends AW_Trigger
{
	public $name = 'guest_abandoned_cart';

	public $group = 'Cart';

	public $allow_queueing = false;

	public $supplied_data_items = array( 'cart', 'guest', 'shop' );


	/**
	 * Construct
	 */
	public function init()
	{
		$this->title = __('Abandoned Cart (Guests)', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$delay = new AW_Field_Number_Input();
		$delay->set_name('delay');
		$delay->set_title( __( 'Hours After Abandoned', 'automatewoo' ) );

		$delay->set_min('0.25');
		$delay->set_max('720'); // 30 days
		$delay->set_description(__('Max is 720 hours (30 days)', 'automatewoo' ));
		$delay->add_extra_attr('step', '0.25');

		$total_greater_than = new AW_Field_Number_Input();
		$total_greater_than->set_name('total_greater_than');
		$total_greater_than->set_title( __( 'Cart Total Greater Than', 'automatewoo' ) );
		$total_greater_than->set_description( __( 'Only trigger when the cart total is greater than a certain amount', 'automatewoo'  ) );

		$total_less_than = new AW_Field_Number_Input();
		$total_less_than->set_name('total_less_than');
		$total_less_than->set_title( __( 'Cart Total Less Than', 'automatewoo' ) );
		$total_less_than->set_description( __( 'Only trigger when the cart total is less than a certain amount', 'automatewoo'  ) );

		$this->add_field( $delay );
		$this->add_field( $total_greater_than );
		$this->add_field( $total_less_than );
		$this->add_field_user_pause_period();
	}



	/**
	 * When should this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'automatewoo_thirty_minute_worker', array( $this, 'catch_hooks' ) );
	}



	/**
	 * Route hooks through here
	 */
	public function catch_hooks()
	{
		if ( ! AutomateWoo()->abandoned_cart->in_use() ) return;

		$carts_query = new AW_Query_Abandoned_Carts();
		$carts_query->where('user_id', 0);
		$carts = $carts_query->get_results();

		if ( ! is_array($carts) )
			return;

		foreach ( $carts as $cart )
		{
			$guest = new AW_Model_Guest( $cart->guest_id );

			// cart must have items and a guest
			if ( ! $cart->items || ! $guest->exists ) continue;

			$this->maybe_run(array(
				'guest' => $guest,
				'cart' => $cart
			));
		}

	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$cart = $workflow->get_data_item('cart');
		$guest = $workflow->get_data_item('guest');

		if ( ! $cart || ! $guest )
			return false;


		$delay = floatval( $trigger->get_option('delay') );
		$total_greater_than = floatval( $trigger->get_option('total_greater_than') );
		$total_less_than = floatval( $trigger->get_option('total_less_than') );
		$cart_total = floatval($cart->total);

		if ( ! $delay ) return false; // delay must be set


		// validate totals
		if ( $total_greater_than )
		{
			if ( $total_greater_than > $cart_total )
				return false;
		}

		if ( $total_less_than )
		{
			if ( $total_less_than < $cart_total )
				return false;
		}


		// convert delay to minutes
		$delay_mins = round( $delay * 60 );

		$delay_date = new DateTime(); // UTC
		$delay_date->setTimestamp( current_time('timestamp', true )  );
		$delay_date->modify("-$delay_mins minutes");


		// Was cart last updated longer than the delay date
		if ( $cart->last_modified > $delay_date->format('Y-m-d H:i:s') )
			return false;


		if ( ! $this->validate_field_user_pause_period( $workflow ) )
			return false;


		// Now check our logs
		// Only run each once foreach workflow for each stored cart
		$log_query = new AW_Query_Logs();
		$log_query->where( 'workflow_id', $workflow->get_translation_ids() );
		$log_query->where( 'cart_id', $cart->id );
		$log_query->set_limit(1);

		if ( $log_query->get_results() )
			return false;

		return true;
	}


}
