<?php
/**
 * @class       AW_Trigger_Abstract_Order_Base
 * @package     AutomateWoo/Triggers
 */

abstract class AW_Trigger_Abstract_Order_Base extends AW_Trigger
{
	public $group = 'Order';

	public $supplied_data_items = array( 'user', 'order', 'shop' );
	

	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type();
		$user_type->set_description( __( 'Only trigger when the order belongs to a user of a certain type', 'automatewoo'  ) );

		$order_total_greater_than = new AW_Field_Number_Input();
		$order_total_greater_than->set_name('order_total_greater_than');
		$order_total_greater_than->set_title( __( 'Order Total Greater Than', 'automatewoo' ) );
		$order_total_greater_than->set_description( __( 'Only trigger when the total order amount is greater than a certain amount', 'automatewoo'  ) );

		$order_total_less_than = new AW_Field_Number_Input();
		$order_total_less_than->set_name('order_total_less_than');
		$order_total_less_than->set_title( __( 'Order Total Less Than', 'automatewoo' ) );
		$order_total_less_than->set_description( __( 'Only trigger when the total order amount is less than a certain amount', 'automatewoo'  ) );

		$is_users_first_order = new AW_Field_Checkbox();
		$is_users_first_order->set_name('is_users_first_order');
		$is_users_first_order->set_title( __('Is Users First Order?', 'automatewoo' ));
		$is_users_first_order->set_description( __( 'Only trigger for a users first order', 'automatewoo'  ) );

		$payment_gateway = new AW_Field_Payment_Gateway();
		$shipping_methods = new AW_Field_Shipping_Methods();

		$shipping_countries = new AW_Field_Countries();
		$shipping_countries->set_name('shipping_countries');
		$shipping_countries->set_title( __('Shipping Countries', 'automatewoo' ));

		$billing_countries = new AW_Field_Countries();
		$billing_countries->set_name('billing_countries');
		$billing_countries->set_title( __('Billing Countries', 'automatewoo' ));


		$this->add_field( $user_type );
		$this->add_field( $order_total_greater_than );
		$this->add_field( $order_total_less_than );
		$this->add_field( $payment_gateway );
		$this->add_field( $shipping_methods );
		$this->add_field( $shipping_countries );
		$this->add_field( $billing_countries );
		$this->add_user_tags_field();
		$this->add_field( $is_users_first_order );
		$this->add_user_limit_field();

		if ( AutomateWoo()->integrations->is_woo_pos() )
		{
			$is_pos = new AW_Field_Select( false );
			$is_pos->set_name( 'is_pos' );
			$is_pos->set_title( __('Is POS?', 'automatewoo' ) );
			$is_pos->set_options(array(
				'' => __('- Any - ', 'automatewoo' ),
				'yes' => __('Yes', 'automatewoo' ),
				'no' => __('No', 'automatewoo' )
			));

			$this->add_field( $is_pos );
		}

	}


	/**
	 * Route hooks through here
	 *
	 * @param $order_id
	 */
	public function catch_hooks( $order_id )
	{
		$order = wc_get_order( $order_id );
		$user = $order->get_user();

		// order placed by a guest
		if ( ! $user )
		{
			$user = new AW_Model_Order_Guest( $order );
		}

		$this->maybe_run(array(
			'order' => $order,
			'user' => $user,
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		/**
		 * @var WP_User $user
		 * @var WC_Order $order
		 */
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order )
			return false;

		// get options
		$shipping_methods = $trigger->get_option('shipping_methods');
		$shipping_countries = $trigger->get_option('shipping_countries');
		$billing_countries = $trigger->get_option('billing_countries');
		$payment_gateway = $trigger->get_option('payment_gateway');

		if ( AutomateWoo()->integrations->is_woo_pos() )
		{
			$pos_option = $trigger->get_option('is_pos');
			$is_pos = get_post_meta( $order->id, '_pos', true );

			if ( $pos_option )
			{
				if ( $pos_option == 'yes' && ! $is_pos )
					return false;

				if ( $pos_option == 'no' && $is_pos )
					return false;
			}
		}

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user limit
		if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
			return false;

		// validate user tags
		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;

		// validate payment gateway
		if ( ! empty( $payment_gateway ) )
		{
			if ( ! in_array( $order->payment_method, $payment_gateway ) )
				return false;
		}

		// validate shipping countries
		if ( ! empty( $shipping_countries ) )
		{
			if ( ! in_array( $order->shipping_country, $shipping_countries ) )
				return false;
		}

		// validate billing countries
		if ( ! empty( $billing_countries ) )
		{
			if ( ! in_array( $order->billing_country, $billing_countries ) )
				return false;
		}


		// validate shipping methods
		if ( ! empty( $shipping_methods ) )
		{
			$has_any = false;

			foreach ( $shipping_methods as $shipping_method )
			{
				if ( $order->has_shipping_method($shipping_method ) )
					$has_any = true;
			}

			// order doesn't match any of the selected methods
			if ( ! $has_any ) return false;
		}


		// Validate order total greater than
		$order_total_greater_than = absint( $trigger->get_option('order_total_greater_than') );
		if ( $order_total_greater_than )
		{
			if ( $order->get_total() < $order_total_greater_than )
				return false;
		}

		// Validate order total less than
		$order_total_less_than = absint( $trigger->get_option('order_total_less_than') );
		if ( $order_total_less_than )
		{
			if ( $order->get_total() > $order_total_less_than )
				return false;
		}

		// validate first order checkbox
		if ( ! $this->validate_is_first_order_field($workflow, $trigger) )
			return false;

		return true;
	}



}
