<?php
/**
 * This trigger hooks in to the daily workflow cron worker and checks to see if a user has not purchased anything in a set period of time.
 *
 * @class       AW_Trigger_User_Absent
 * @package     AutomateWoo/Triggers
 * @since       1.0.0
 */

class AW_Trigger_User_Absent extends AW_Trigger
{
	public $name = 'user_absent';

	public $group = 'User';

	public $allow_queueing = false;

	public $supplied_data_items = array( 'user', 'shop' );


	public function init()
	{
		$this->title = __('User Has Not Purchased For a Set Period', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type();
		$user_type->set_description( __( 'Only trigger for users of a certain type.', 'automatewoo'  ) );

		$period = new AW_Field_Number_Input();
		$period->set_name('days_since_last_purchase');
		$period->set_title( __( 'Days Since Last Purchase', 'automatewoo' ) );
		$period->set_description( __( 'This will trigger once when the period ends...', 'automatewoo'  ) );

		$this->add_field( $period );
		$this->add_field( $user_type );
		$this->add_user_tags_field();
		$this->add_user_limit_field();
	}



	/**
	 * When should this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'automatewoo_daily_worker', array( $this, 'catch_hooks' ) );
	}



	/**
	 * Route hooks through here
	 */
	public function catch_hooks()
	{
		// As this query is going to be memory intensive lets make sure we have a workflow using this trigger
		if ( ! $this->has_workflows() )
			return;

		// Needs to be tested against each user
		$users = get_users();

		foreach ( $users as $user )
		{
			$this->maybe_run(array(
				'user' => $user
			));
		}

	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');

		if ( ! $user )
			return false;

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user limit
		if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
			return false;

		// validate user tags
		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;


		// Make sure user has not made a purchase in the last set period of days
		// Also need to make sure we haven't run the workflow in the last set period of days
		$period = absint( $trigger->get_option('days_since_last_purchase') );

		if ( $period )
		{
			$date = new DateTime( current_time('mysql', true ) );
			$date->modify("-$period days");

			// Fail if the user has not been a member for at least the set period
			if ( strtotime( $user->user_registered ) > $date->getTimestamp() )
				return false;


			// Get any orders after the set period
			$orders_query = new WP_Query(array(
				'posts_per_page' => 1,
				'post_type' => 'shop_order',
				'post_status' => 'any',
				'fields' => 'ids',
				'date_query' => array(
					array(
						'column' => 'post_date_gmt',
						'after' => $date->format('Y-m-d H:i:s')
					)
				),
				'meta_query' => array(
					array(
						'key' => '_customer_user',
						'value' => $user->ID,
					)
				),
			));

			// User has ordered is there are posts
			if ( $orders_query->have_posts() )
				return false;


			// Now check our logs for the last run
			$log_query = new AW_Query_Logs();

			$log_query->where( 'workflow_id', $workflow->id );
			$log_query->where( 'user_id', $user->ID );
			$log_query->where( 'date', $date, '>' );
			$log_query->set_limit(1);

			if ( $log_query->get_results() )
				return false;
		}


		return true;

	}


}
