<?php
/***
 * @class       AW_Trigger_User_Leaves_Review
 * @package     AutomateWoo/Triggers
 * @since       1.0.0
 */

class AW_Trigger_User_Leaves_Review extends AW_Trigger
{
	public $name = 'user_leaves_review';

	public $group = 'User';

	public $supplied_data_items = array( 'user', 'product', 'shop', 'comment' );

	public function init()
	{
		$this->title = __('User Leaves a Product Review', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger when the review is made by a user of a certain type.', 'automatewoo'  ) );

		$this->add_field( $user_type );
		$this->add_user_limit_field();
	}



	/**
	 * When could this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'transition_comment_status', array( $this, 'catch_comment_approval' ), 20, 3 );
		add_action( 'comment_post', array( $this, 'catch_new_comments' ), 10, 2 );
	}


	/**
	 * Catch any comments approved on creation
	 *
	 * @param $comment_ID
	 * @param $approved
	 */
	public function catch_new_comments( $comment_ID, $approved )
	{
		if ( $approved != 1 )
			return;

		$comment = get_comment( $comment_ID );

		$this->catch_hooks( $comment );
	}


	/**
	 * Catch any comments that were approved after creation
	 *
	 * @param $new_status string
	 * @param $old_status string
	 * @param $comment object
	 */
	public function catch_comment_approval( $new_status, $old_status, $comment )
	{
		if ( $new_status !== 'approved' )
			return;

		$this->catch_hooks( $comment );
	}


	/**
	 *
	 * @param $comment object
	 */
	public function catch_hooks( $comment )
	{
		if ( ! $comment->user_id )
			return;

		// Make sure the comment is on a product
		if ( 'product' === get_post_type( $comment->comment_post_ID ) )
		{
			$user = get_user_by( 'id', $comment->user_id );
			$product = wc_get_product( $comment->comment_post_ID );

			$this->maybe_run(array(
				'user' => $user,
				'product' => $product,
				'comment' => $comment
			));
		}
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$product = $workflow->get_data_item('product');
		$comment = $workflow->get_data_item('comment');

		if ( ! $user || ! $product || ! $comment )
			return false;

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user limit
		if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
			return false;

		// only run once for each comment and workflow
		// just in case the comment is approved more than once
		$log_query = new AW_Query_Logs();
		$log_query->set_limit(1);
		$log_query->where( 'workflow_id', $workflow->id );
		$log_query->where( 'comment_id', $comment->comment_ID );

		if ( $log_query->get_results() )
			return false;

		return true;
	}


}
