<?php
/**
 * Only allows a single product choice as the text variable system only supports single data items
 *
 * @class       AW_Trigger_User_Purchases_Specific_Product
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_User_Purchases_Specific_Product extends AW_Trigger
{

	/**
	 * @var string
	 */
	public $title;


	/**
	 * @var string
	 */
	public $name = 'user_purchases_specific_product';


	/**
	 * @var string
	 */
	public $group = 'Order';


	/**
	 * Data items that will be passed to the action
	 * @var array
	 */
	public $supplied_data_items = array( 'user', 'order', 'product', 'shop' );


	/**
	 * Construct
	 */
	public function init()
	{
		$this->title = __('Order Includes a Specific Product', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$product = new AW_Field_Product();
		$product->allow_variations = true;
		$product->set_description( __( 'Only trigger when a certain product is purchased.', 'automatewoo'  ) );
		$product->set_required(true);

		$user_type = new AW_Field_User_Type();
		$user_type->set_description( __( 'Only trigger when the purchase is made by a user of a certain type.', 'automatewoo'  ) );

		$order_status = new AW_Field_Order_Status( false );
		$order_status->set_required(true);
		$order_status->set_default('wc-completed');

		$this->add_field( $product );
		$this->add_field( $order_status );
		$this->add_field_validate_queued_order_status();
		$this->add_field( $user_type );
		$this->add_user_tags_field();
		$this->add_user_limit_field();
	}



	/**
	 * When could this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'woocommerce_order_status_changed', array( $this, 'catch_hooks' ), 100, 1 );
	}


	/**
	 * Route hooks through here
	 *
	 * @param $order_id
	 */
	public function catch_hooks( $order_id )
	{
		$order = wc_get_order( $order_id );
		$user = $order->get_user();

		// order placed by a guest
		if ( ! $user )
		{
			$user = new AW_Model_Order_Guest($order);
		}

		$this->maybe_run(array(
			'order' => $order,
			'user' => $user,
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order )
			return false;

		if ( ! $this->validate_order_status_field( $trigger, $order ) )
			return false;

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user limit
		if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
			return false;

		// validate user tags
		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;


		$valid_product = wc_get_product( absint( $trigger->get_option('product') ) );

		// Validate product
		if ( $valid_product )
		{
			$id_key = $valid_product->get_type() == 'variation' ? 'variation_id' : 'product_id';

			foreach ( $order->get_items() as $item )
			{
				if ( $item[$id_key] > 0 )
				{
					if ( $item[$id_key] == $valid_product->get_id() )
					{
						// user has bought something the selected product

						// Pass data item through to workflow
						$workflow->add_data_item( 'product', $valid_product );
						return true;
					}
				}
			}
		}

		return false;
	}



	/**
	 * Ensures 'to' status has not changed while sitting in queue
	 *
	 * @param $workflow
	 *
	 * @return bool
	 */
	public function validate_before_queued_event( $workflow )
	{
		// check parent
		if ( ! parent::validate_before_queued_event( $workflow ) )
			return false;

		$trigger = $workflow->get_trigger();
		$order = $workflow->get_data_item('order');

		if ( ! $order )
			return false;

		// Option to validate order status
		if ( $trigger->get_option('validate_order_status_before_queued_run') )
		{
			if ( ! $this->validate_status_field( $trigger->get_option('order_status'), $order->get_status() ) )
				return false;
		}

		return true;
	}


}
