<?php
/**
 * This trigger hooks in the the order completed action but will only fire once when a users total spend reaches a certain amount.
 *
 * @class       AW_Trigger_User_Total_Spend_Reaches
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_User_Total_Spend_Reaches extends AW_Trigger
{
	public $name = 'users_total_spend';

	public $group = 'User';

	public $supplied_data_items = array( 'user', 'shop' );

	public function init()
	{
		$this->title = __('User Total Spend Reaches', 'automatewoo');
		$this->description = __("This trigger checks the user's total spend each time an order is completed. Note that it does not work for guests.", 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger for users of a certain type.', 'automatewoo'  ) );

		$total_spend = new AW_Field_Number_Input();
		$total_spend->set_name( 'total_spend' );
		$total_spend->set_title( __( 'Total Spend', 'automatewoo' ) );
		$total_spend->set_description( __( 'No need to add a currency symbol.', 'automatewoo'  ) );
		$total_spend->set_required(true);

		$this->add_field( $total_spend );
		$this->add_field( $user_type );
		$this->add_user_tags_field();
	}



	/**
	 * Must run on complete after WC has updated its records
	 */
	public function register_hooks()
	{
		add_action( 'woocommerce_order_status_completed', array( $this, 'catch_hooks' ), 100, 1 );
	}



	/**
	 * Route hooks through here
	 */
	public function catch_hooks( $order_id )
	{
		$order = wc_get_order( $order_id );
		$user = $order->get_user();

		if ( ! $user ) return; // doesn't work for guests

		$this->maybe_run(array(
			'order' => $order,
			'user' => $user
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order )
			return false;

		// Validate User type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// Only do this once for each user (for each workflow)
		if ( ! $workflow->is_first_run_for_user( $user ) )
			return false;

		// validate user tags
		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;

		// Validate total spent
		$total_spend = floatval( $trigger->get_option('total_spend') );

		if ( $total_spend )
		{
			if ( floatval( wc_get_customer_total_spent( $user->ID ) ) <= $total_spend )
				return false;
		}

		return true;
	}

}
