<?php
/**
 * @class       AW_Trigger_Wishlist_Item_Goes_On_Sale
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_Wishlist_Item_Goes_On_Sale extends AW_Trigger
{
	public $name = 'wishlist_item_goes_on_sale';

	public $group = 'Wishlist';

	public $supplied_data_items = array( 'user', 'shop', 'product', 'wishlist' );


	/**
	 * Construct
	 */
	public function init()
	{
		$integration = AutomateWoo()->wishlist->get_integration();

		// don't enable triggers
		if ( ! $integration )
			return;

		$this->title = sprintf( __('Product in a Wishlist goes on Sale (%s)', 'automatewoo'), AutomateWoo()->wishlist->get_integration_title() );
		$this->description = __(
			"This trigger can't fire immediately when a product goes on sale so instead it performs a check every four hours. "
			. "Please note this doesn't work for guests because their wishlist data only exists in their session data.",
			'automatewoo');

		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger for users of a certain type.', 'automatewoo'  ) );

		$this->add_field( $user_type );
	}



	/**
	 * When might this trigger run?
	 */
	public function register_hooks()
	{
		$integration = AutomateWoo()->wishlist->get_integration();

		if ( ! $integration )
			return;

		add_action( 'automatewoo_four_hourly_worker', array( $this, 'catch_hooks_' . $integration ) );
	}



	/**
	 * Route hooks through here
	 */
	public function catch_hooks_woothemes()
	{
		if ( ! $this->has_workflows() )
			return;

		$products_on_sale = wc_get_product_ids_on_sale();

		$wishlists = get_posts(array(
			'post_type' => 'wishlist',
			'fields' => 'ids'
		));

		if ( is_array( $wishlists ) )
		{
			foreach( $wishlists as $list_id )
			{
				$products = get_post_meta( $list_id, '_wishlist_items', true );

				if ( ! $products )
					continue;

				foreach( $products as $product )
				{
					if ( in_array( $product['product_id'], $products_on_sale ) )
					{
						$user_id = get_post_meta( $list_id, '_wishlist_owner', true );
						$user = get_user_by( 'id', $user_id );

						$_product = wc_get_product( $product['product_id'] );

						$this->maybe_run(array(
							'user' => $user,
							'product' => $_product
						));
					}
				}
			}
		}
	}


	/**
	 * Route hooks through here
	 */
	public function catch_hooks_yith()
	{
		if ( ! $this->has_workflows() )
			return;

		$products_on_sale = wc_get_product_ids_on_sale();

		$wishlists = YITH_WCWL()->get_wishlists();

		if ( is_array( $wishlists ) )
		{
			foreach( $wishlists as $wishlist )
			{
				$products = YITH_WCWL()->get_products(array(
					'wishlist_id' => $wishlist['ID']
				));

				if ( ! $products )
					continue;

				foreach( $products as $product )
				{
					if ( in_array( $product['prod_id'], $products_on_sale ) )
					{
						$user = get_user_by( 'id', $wishlist['user_id'] );

						$_product = wc_get_product( $product['prod_id'] );

						$this->maybe_run(array(
							'user' => $user,
							'product' => $_product
						));
					}
				}
			}
		}
	}



	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$product = $workflow->get_data_item('product');


		if ( ! $user || ! $product )
			return false;

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;


		// Only trigger once per user, per product, per workflow, check logs
		$log_query = new AW_Query_Logs();

		$log_query->where( 'workflow_id', $workflow->id );
		$log_query->where( 'product_id', $product->id );
		$log_query->where( 'user_id', $user->ID );

		if ( $log_query->get_results() )
			return false;

		return true;
	}

}

