<?php

/**
 * Variable Processor
 *
 * Process variables into values. Is used on workflows and action options.
 *
 * @class AW_Variable_Processor
 * @since 2.0.2
 */
class AW_Variable_Processor
{

	/**
	 * @var AW_Model_Workflow
	 */
	public $workflow;


	/**
	 * @param $workflow
	 */
	public function __construct( $workflow )
	{
		$this->workflow = $workflow;
	}



	/**
	 *
	 * @param $text string
	 * @param bool $allow_html
	 *
	 * @return string
	 */
	public function process_field( $text, $allow_html = false )
	{
		$value = preg_replace_callback('~\{\{(.*?)\}\}~', array( $this,'_process_preg_callback' ), $text );

		if ( ! $allow_html )
		{
			$value = strip_tags( $value );
		}

		return $value;
	}


	/**
	 * @param $match
	 *
	 * @return bool|mixed
	 */
	function _process_preg_callback($match)
	{
		$parameters_array = array();

		$var = $this->sanitize( $match[1] );

		if ( ! strstr( $var, '.' ) )
			return false;

		// Only explode once so a full stop can be used in params
		list( $data_type, $data_value ) = explode('.', $var, 2 );


		// Does this have parameters?
		if ( strstr( $var, '|' ) )
		{
			// extract parameters
			list( $data_value, $parameters ) = explode( '|', $data_value );

			$parameters = explode( ',', $parameters );

			foreach ( $parameters as $parameter )
			{
				if ( strstr( $parameter, ':' ) )
				{
					list( $key, $value ) = explode( ':', $parameter );
					$parameters_array[sanitize_title($key)] = sanitize_text_field($value);
				}
			}
		}


		$data_type = sanitize_title( $data_type );
		$data_value = sanitize_title( $data_value );

		$value = $this->get_value( $data_type, $data_value, $parameters_array );

		if ( ! $value )
		{
			// backwards compatibility
			if ( isset( $parameters_array['default'] ) )
				$parameters_array['fallback'] = $parameters_array['default'];

			// show default if set and no real value
			if ( isset( $parameters_array['fallback'] ) )
				$value = $parameters_array['fallback'];
		}

		return $value;

	}



	/**
	 * @param $data_type
	 * @param $value
	 * @param $parameters
	 *
	 * @return mixed
	 *
	 *
	 */
	public function get_value( $data_type, $value, $parameters = array() )
	{
		/**
		 * @var $user WP_User
		 * @var $order WC_Order
		 * @var $comment WP_Comment
		 * @var $guest AW_Model_Guest
		 * @var $subscription WC_Subscription
		 */

		if ( $filtered = apply_filters('automatewoo_text_variable_value', false, $data_type, $value ) )
			return $filtered;

		$this->_compatibility( $data_type, $value, $parameters );


		switch ( $data_type )
		{
			case 'user':

				if ( ! $user = $this->workflow->get_data_item('user') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $user->ID;
						break;

					case 'email':
						return $user->user_email;
						break;

					case 'firstname':
						// todo update to user_firstname also need to update AW_Model_Order_Guest
						if ( $user->first_name )
						{
							return $user->first_name;
						}
						elseif ( $order = $this->workflow->get_data_item('order') )
						{
							return $order->billing_first_name;
						}

						break;

					case 'lastname':
						// todo update to user_lastnamename
						if ( $user->last_name )
						{
							return $user->last_name;
						}
						elseif ( $order = $this->workflow->get_data_item('order') )
						{
							return $order->billing_last_name;
						}

						break;

					case 'username':
						return $user->user_login;
						break;


					/**
					 * Field parameters
					 *
					 * key
					 */
					case 'meta':

						if ( empty( $parameters['key'] ) )
							return false;

						return get_user_meta($user->ID, $parameters['key'], true );

						break;


					/**
					 * Generate a coupon unique to a user but based on an existing coupon
					 *
					 * Field parameters: template, prefix, expires, chance
					 */
					case 'generate_coupon':

						return $this->generate_coupon( $user->user_email, $parameters );

						break;
				}

				break;


			case 'order':

				if ( ! $order = $this->workflow->get_data_item('order') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $order->id;
						break;

					case 'number':
						return $order->get_order_number();
						break;

					case 'status':
						return $order->get_status();
						break;

					case 'total':
						return $order->get_formatted_order_total();
						break;

					case 'itemscount':
						return $order->get_item_count();
						break;

					case 'billing_phone':
						return $order->billing_phone;
						break;

					case 'payment_url':
						return $order->get_checkout_payment_url();
						break;

					case 'view_url':
						return $order->get_view_order_url();
						break;

					case 'items':

						$items = $order->get_items();
						$products = array();
						$template = false;

						foreach ( $items as $item )
						{
							$products[] = apply_filters( 'woocommerce_order_item_product', $order->get_product_from_item( $item ), $item );
						}

						if ( isset( $parameters['template'] ) )
							$template = $parameters['template'];

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type
						));

						break;


					case 'cross_sells':

						$cross_sells = array();
						$in_order = array();
						$template = false;

						$items = $order->get_items();

						foreach ( $items as $item )
						{
							$product = apply_filters( 'woocommerce_order_item_product', $order->get_product_from_item( $item ), $item );
							$in_order[] = $product->id;
							$cross_sells = array_merge( $product->get_cross_sells(), $cross_sells );
						}

						$cross_sells = array_diff( $cross_sells, $in_order );

						if ( empty( $cross_sells ) ) return;

						$products = $this->_get_products( $cross_sells, apply_filters( 'automatewoo_cross_sells_total', 8 ) );

						if ( isset( $parameters['template'] ) )
							$template = $parameters['template'];

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type
						));

						break;


					case 'related_products':

						$related = array();
						$in_order = array();
						$template = false;

						$items = $order->get_items();

						foreach ( $items as $item )
						{
							$product = apply_filters( 'woocommerce_order_item_product', $order->get_product_from_item( $item ), $item );
							$in_order[] = $product->id;
							$related = array_merge( $product->get_related(), $related );
						}

						$related = array_diff( $related, $in_order );

						if ( empty( $related ) ) return;

						$products = $this->_get_products( $related, apply_filters( 'automatewoo_related_total', 8 ) );

						if ( isset( $parameters['template'] ) )
							$template = $parameters['template'];

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type
						));

						break;


					case 'meta':
						if ( $parameters['key'] )
							return get_post_meta($order->id, $parameters['key'], true );
						break;


					case 'tracking_number':

						if ( ! $item = $this->get_shipment_tracking_item() )
							return false;

						return $item[ 'tracking_number' ];

						break;


					case 'tracking_url':

						if ( ! $item = $this->get_shipment_tracking_item() )
							return false;

						return $item[ 'formatted_tracking_link' ];

						break;

					case 'date_shipped':

						if ( ! $item = $this->get_shipment_tracking_item() )
							return false;

						return $item[ 'date_shipped' ];

						break;

					case 'shipping_provider':

						if ( ! $item = $this->get_shipment_tracking_item() )
							return false;

						return $item[ 'formatted_tracking_provider' ];

						break;


				}

				break;


			case 'product':

				if ( ! $product = $this->workflow->get_data_item('product') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $product->id;
						break;

					case 'title':
						return $product->get_title();
						break;

					case 'permalink':
						return $product->get_permalink();
						break;

					case 'add_to_cart_url':
						return $product->get_permalink() . '?add-to-cart=' . $product->id;
						break;

					case 'current_price':
						return wc_price( $product->get_price() );
						break;

					case 'regular_price':
						return wc_price( $product->get_regular_price() );
						break;

					case 'featured_image':
						return $product->get_image('shop_catalog');
						break;

					case 'sku':
						return $product->get_sku();
						break;

					case 'short_description':
						return $product->post->post_excerpt;
						break;

					case 'meta':
						if ( $parameters['key'] )
							return get_post_meta($product->id, $parameters['key'], true );
						break;

				}

				break;


			case 'category':

				if ( ! $category = $this->workflow->get_data_item('category') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $category->term_id;
						break;

					case 'title':
						return $category->name;
						break;

					case 'permalink':
						$link = get_term_link( $category );
						if ( ! $link instanceof WP_Error )
							return $link;
						break;
				}

				break;

			case 'wishlist':

				if ( ! $wishlist = $this->workflow->get_data_item('wishlist') )
					return false;

				switch ( $value )
				{
					case 'view_link':

						return $wishlist->permalink;

						break;

					case 'items':

						$products = array();
						$template = false;

						foreach ( $wishlist->items as $product_id )
						{
							$products[] = wc_get_product( $product_id );
						}

						if ( isset( $parameters['template'] ) )
							$template = $parameters['template'];

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type
						));

						break;


					case 'itemscount':

						if ( ! is_array( $wishlist->items ) )
							return 0;

						return count( $wishlist->items );

						break;

				}

				break;


			case 'cart':

				if ( ! $cart = $this->workflow->get_data_item('cart') )
					return false;

				switch ( $value )
				{
					case 'link':

						return add_query_arg(array(
							'aw-restore-cart' => $cart->id
						), wc_get_page_permalink('cart') );

						break;


					case 'total':
						return wc_price( $cart->total );
						break;

					case 'items':

						$cart_items = maybe_unserialize($cart->items);
						$template = false;

						$products = array();
						$product_ids = array();

						if ( ! is_array($cart_items) )
							return;


						foreach ( $cart_items as $item )
						{
							if ( isset( $item['product_id'] ) )
								$product_ids[] = $item['product_id'];
						}

						$product_ids = array_unique( $product_ids );

						foreach ( $product_ids as $product_id )
						{
							$products[] = wc_get_product( $product_id );
						}


						if ( isset( $parameters['template'] ) )
							$template = $parameters['template'];

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type,
							'cart_items' => $cart_items
						));

						break;
				}

				break;


			case 'guest':

				if ( ! $guest = $this->workflow->get_data_item('guest') )
					return false;


				switch ( $value )
				{
					case 'email':

						return $guest->email;

						break;

					/**
					 * Generate a coupon unique to a user but based on an existing coupon
					 *
					 * Field parameters: template, prefix, expires
					 */
					case 'generate_coupon':

						return $this->generate_coupon( $guest->email, $parameters );

						break;
				}

				break;


			case 'order_item':

				if ( ! $order_item = $this->workflow->get_data_item('order_item') )
					return false;

				switch ( $value )
				{
					case 'attribute':

						// requires a slug
						if ( empty( $parameters['slug'] ) )
							return false;

						$attribute = 'pa_' . $parameters['slug'];

						if ( empty( $order_item[$attribute] ) )
							return false;

						$term = $order_item[$attribute];

						$term_obj = get_term_by('slug', $term, $attribute );

						if ( ! $term_obj || is_wp_error($term_obj) )
							return false;

						return $term_obj->name;

						break;
				}

				break;



			case 'subscription':

				if ( ! $subscription = $this->workflow->get_data_item('subscription') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $subscription->id;
						break;

					case 'status':
						return $subscription->get_status();
						break;

					case 'payment_method':
						return $subscription->get_payment_method_to_display();
						break;

					case 'view_order_url':
						return $subscription->get_view_order_url();
						break;

					case 'start_date':
					case 'next_payment_date':
					case 'trial_end_date':
					case 'end_date':
					case 'last_payment_date':

						$date_field = str_replace('_date','',$value );
						$date = $subscription->get_date( $date_field, 'site');

						if ( ! $date ) return false;

						$datetime = new DateTime();
						$datetime->setTimestamp( strtotime( $date ) );
						return $this->datetime( $datetime, $parameters );
						break;

				}

				break;



			case 'comment':

				if ( ! $comment = $this->workflow->get_data_item('comment') )
					return false;

				switch ( $value )
				{
					case 'id':
						return $comment->comment_ID;
						break;

					case 'author_name':
						return $comment->comment_author;
						break;

					case 'author_ip':
						return $comment->comment_author_IP;
						break;

				}


				break;


			case 'shop':

				switch ( $value )
				{
					case 'title':
						return get_bloginfo('name');
						break;

					case 'tagline':
						return get_bloginfo('description');
						break;

					case 'url':
						return home_url();
						break;

					case 'admin_email':
						return get_bloginfo('admin_email');
						break;

					case 'current_datetime':
						$datetime = new DateTime();
						$datetime->setTimestamp( current_time('timestamp') );
						return $this->datetime( $datetime, $parameters );
						break;

					case 'products':

						// params
						$type = isset( $parameters['type'] ) ? $parameters['type'] : false;
						$template = isset( $parameters['template'] ) ? $parameters['template'] : false;
						$limit = isset( $parameters['limit'] ) ? absint( $parameters['limit'] ) : 8;

						switch ( $type )
						{
							case 'ids':

								if ( empty( $parameters['ids'] ) )
									return false;

								$ids = explode('+', $parameters['ids'] );
								$ids = array_map( 'absint', $ids );

								$products = $this->_get_products( $ids, $limit );

								break;

							case 'category':

								if ( empty( $parameters['category'] ) )
									return false;

								$category = get_term_by( 'slug', $parameters['category'], 'product_cat' );

								if ( ! $category )
									return false;

								$products = $this->_get_products( get_objects_in_term( $category->term_id, 'product_cat' ), $limit );

								break;

							case 'tag':

								if ( empty( $parameters['tag'] ) )
									return false;

								$tag = get_term_by( 'slug', $parameters['tag'], 'product_tag' );

								if ( ! $tag )
									return false;

								$products = $this->_get_products( get_objects_in_term( $tag->term_id, 'product_tag' ), $limit );

								break;

							case 'featured':
								$products = $this->_get_products( wc_get_featured_product_ids(), $limit );
								break;

							case 'sale':
								$products = $this->_get_products( wc_get_product_ids_on_sale(), $limit );
								break;

							case 'recent':
								$products = $this->_get_products( aw_get_recent_product_ids( $limit ), $limit );
								break;

							case 'custom':

								if ( empty( $parameters['filter'] ) )
									return false;

								$product_ids = apply_filters( $parameters['filter'], array(), $this->workflow, $parameters );
								$products = $this->_get_products( $product_ids, $limit );

								break;

							default:
								return false;
							break;
						}

						return $this->get_email_product_list( $template, array(
							'products' => $products,
							'data_type' => $data_type
						));

						break;

				}

				break;
		}
	}



	/**
	 * @param string $template
	 * @param array $args
	 *
	 * @return string
	 */
	private function get_email_product_list( $template, $args = array() )
	{
		ob_start();

		if ( $template )
		{
			$template = sanitize_file_name( $template );
		}
		else
		{
			$template = 'product-grid-2-col.php';
		}

		aw_get_template( 'email/' . $template, $args );

		return ob_get_clean();
	}


	/**
	 * Based on sanitize_title()
	 *
	 * @param $string
	 *
	 * @return mixed|string
	 */
	public function sanitize( $string )
	{
		// remove style and script tags
		$string = wp_strip_all_tags( $string, true );
		$string = remove_accents( $string );

		// remove disallowed chars
		$remove = array(
			'{', '}', '[', ']', '=',
		);

		$string = str_replace( $remove, '', $string );

		// remove unicode white spaces
		$string = preg_replace( "#\x{00a0}#siu", ' ', $string );

		$string = trim($string);

		return $string;
	}


	/**
	 * @param $email string
	 * @param $parameters array
	 *
	 * @return bool|string
	 */
	private function generate_coupon( $email, $parameters )
	{
		// requires a template
		if ( empty( $parameters['template'] ) )
			return false;

		$coupon = new AW_Coupon();
		$coupon->set_template_coupon_code( $parameters['template'] );


		// override with parameter
		if ( isset( $parameters['prefix'] ) )
			$coupon->set_prefix( $parameters['prefix'] );


		$coupon->generate_code();

		// don't generate a new coupon every time we preview
		if ( $this->workflow->preview_mode )
		{
			return $coupon->code;
		}


		$coupon->set_email_restriction( $email );

		if ( ! empty( $parameters['expires'] ) )
		{
			$coupon->set_expires( $parameters['expires'] );
		}

		if ( $coupon->generate_coupon() )
		{
			return $coupon->code;
		}
	}




	/**
	 * @param DateTime $date
	 * @param array $parameters (modify, format)
	 *
	 * @return string|void
	 */
	private function datetime( $date, $parameters, $format = 'Y-m-d H:i:s' )
	{
		if ( ! $date instanceof DateTime ) return;

		if ( ! empty($parameters['format'] ) )
		{
			$format = $parameters['format'];
		}

		if ( ! empty($parameters['modify'] ) )
		{
			$date->modify( $parameters['modify'] );
		}

		return $date->format($format);
	}


	/**
	 * Gets the first shipment tracking array
	 *
	 * @return array|false
	 */
	private function get_shipment_tracking_item()
	{
		/** @var $order WC_Order */
		if ( ! $order = $this->workflow->get_data_item('order') )
			return false;

		if ( ! class_exists( 'WC_Shipment_Tracking' ) ) return false;

		$tracking_items = WC_Shipment_Tracking_Actions::get_instance()->get_tracking_items( $order->id, true );

		if ( empty( $tracking_items ) ) return false;

		return $tracking_items[0];
	}


	/**
	 * @param $product_ids
	 * @param $limit
	 * @return array
	 */
	private function _get_products( $product_ids, $limit )
	{
		if ( empty( $product_ids ) )
			return false;

		$products = array();

		$args = array(
			'post_type'           => 'product',
			'ignore_sticky_posts' => 1,
			'no_found_rows'       => 1,
			'posts_per_page'      => $limit,
			'post__in'            => $product_ids,
			'meta_query'          => WC()->query->get_meta_query(),
			'fields' => 'ids'
		);

		$product_ids = get_posts( $args );

		foreach ( $product_ids as $product_id )
		{
			$products[] = wc_get_product($product_id);
		}

		return $products;
	}


	/**
	 * Backwards compatibility
	 */
	private function _compatibility( &$data_type, &$value, &$parameters )
	{
		if ( $data_type == 'site' )
		{
			$data_type = 'shop';
		}


		if ( $data_type == 'shop' )
		{
			if ( $value == 'products_on_sale' )
			{
				$value = 'products';
				$parameters['type'] = 'sale';
			}

			if ( $value == 'products_recent' )
			{
				$value = 'products';
				$parameters['type'] = 'recent';
			}

			if ( $value == 'products_featured' )
			{
				$value = 'products';
				$parameters['type'] = 'featured';
			}
		}


		switch ( $data_type )
		{
			case 'site':

				$data_type = 'shop';

				break;
		}
	}




}


